@@ -2,19 +2,23 @@
 use strict;
 
 use Config;
+use Carp;
 use Cwd;
 use File::Spec;
 use File::Path;
 use File::Copy;
 use Data::Dumper;
-use FindBin;
-use lib File::Spec->catdir( $FindBin::Bin, 'lib' );
-use lib $FindBin::Bin;
-use Test::Smoke::Util qw( do_pod2usage );
+use File::Basename;
+my $findbin;
+BEGIN { $findbin = dirname $0 }
+use lib File::Spec->catdir( $findbin, 'lib' );
+use lib $findbin;
+use Test::Smoke::Util qw( do_pod2usage whereis );
+use Test::Smoke::SysInfo;
 
-# $Id: configsmoke.pl 705 2004-05-04 20:08:07Z abeltje $
+# $Id: configsmoke.pl 1032 2007-04-01 20:09:02Z abeltje $
 use vars qw( $VERSION $conf );
-$VERSION = '0.042';
+$VERSION = '0.064';
 
 use Getopt::Long;
 my %options = ( 
@@ -96,6 +100,7 @@ Current options:
 =cut
 
 sub is_win32() { $^O eq 'MSWin32' }
+sub is_vms()   { $^O eq 'VMS'     }
 
 my %config = ( perl_version => $conf->{perl_version} || '5.9.x' );
 
@@ -104,6 +109,7 @@ my @mailers = sort keys %mailers;
 my @syncers = get_avail_sync();
 my $syncmsg = join "\n", @{ { 
     rsync    => "\trsync - Use the rsync(1) program [preferred]",
+    ftp      => "\tftp - Use Net::FTP to sync from APC [!slow!]",
     copy     => "\tcopy - Use File::Copy to copy from a local directory",
     hardlink => "\thardlink - Copy from a local directory using link()",
     snapshot => "\tsnapshot - Get a snapshot using Net::FTP (or LWP::Simple)",
@@ -111,52 +117,75 @@ my $syncmsg = join "\n", @{ {
 my @untars = get_avail_tar();
 my $untarmsg = join "", map "\n\t$_" => @untars;
 
+my %vdirs = map {
+    my $vdir = $_;
+    is_vms and $vdir =~ tr/.//d;
+    ( $_ => $vdir )
+} qw( 5.5.x 5.8.x ); # unsupported: 5.6.2
+
 my %versions = (
-    '5.5.x' => { source => 'ftp.linux.activestate.com::perl-5.005xx',
-                 server => 'ftp.funet.fi',
-                 sdir   => '/pub/languages/perl/snap/5.5.x',
-                 sfile  => '',
-                 pdir   => '/pub/staff/gsar/APC/perl-5.005xx-diffs',
-                 cfg    => 'perl55x.cfg',
-                 ddir   => File::Spec->rel2abs( 
-                               File::Spec->catdir( File::Spec->updir,
-                                                   'perl-5.5.x' ) ),
-                 text   => 'Perl 5.005 MAINT',
-                 is56x  => 1},
-    '5.6.2' => { source => 'ftp.linux.activestate.com::perl-5.6.2',
-                 server => 'http://rgarciasuarez.free.fr',
-                 sdir   => '/snap',
-                 sfile  => 'perl562-21463.tar.gz',
-                 pdir   => '/pub/staff/gsar/APC/perl-5.6.2-diffs',
-                 ddir   => File::Spec->rel2abs( 
-                               File::Spec->catdir( File::Spec->updir,
-                                                   'perl-5.6.2' ) ),
-                 cfg    => 'perl562.cfg',
-                 text   => 'Perl 5.6.2-to-be',
-                 is56x  => 1 },
-    '5.8.x' => { source =>  'ftp.linux.activestate.com::perl-5.8.x',
-                 server => 'ftp.funet.fi',
-                 sdir   => '/pub/languages/perl/snap/5.8.x',
-                 sfile  => '',
-                 pdir   => '/pub/staff/gsar/APC/perl-5.8.x-diffs',
-                 ddir   => File::Spec->rel2abs( 
-                               File::Spec->catdir( File::Spec->updir,
-                                                   'perl-5.8.x' ) ),
+#    '5.5.x' => { source => 'public.activestate.com::perl-5.005xx',
+#                 server => 'public.activestate.com',
+#                 sdir   => '/pub/apc/perl-5.005xx-snap',
+#                 sfile  => '',
+#                 pdir   => '/pub/apc/perl-5.005xx-diffs',
+#                 cfg    => 'perl55x.cfg',
+#                 ddir   => File::Spec->catdir( cwd(), File::Spec->updir,
+#                                               "perl-$vdirs{'5.5.x'}" ),
+#                 text   => 'Perl 5.005 MAINT',
+#                 is56x  => 1},
+
+#    '5.6.2' => { source => 'public.activestate.com::perl-5.6.2',
+#                 server => 'public.activestate.com',
+#                 sdir   => '/pub/apc/perl-5.6.2-snap',
+#                 sfile  => 'perl-5.6.2-latest.tar.gz',
+#                 pdir   => '/pub/apc/perl-5.6.2-diffs',
+#                 ddir   => File::Spec->catdir( cwd(), File::Spec->updir,
+#                                               "perl-$vdirs{'5.6.2'}" ),
+#                 ftphost => 'public.activestate.com',
+#                 ftpusr  => 'anonymous',
+#                 ftppwd  => 'smokers@perl.org',
+#                 ftpsdir => '/pub/apc/perl-5.6.2',
+#                 ftpcdir => '/pub/apc/perl-5.6.2-diffs',
+#
+#                 text   => 'Perl 5.6.2 (final?)',
+#                 cfg    => 'perl562.cfg',
+#                 is56x  => 1 },
+
+    '5.8.x' => { source =>  'public.activestate.com::perl-5.8.x',
+                 server => 'public.activestate.com',
+                 sdir   => '/pub/apc/perl-5.8.x-snap',
+                 sfile  => 'perl-5.8.x-latest.tar.gz',
+                 pdir   => '/pub/apc/perl-5.8.x-diffs',
+                 ddir   => File::Spec->catdir( cwd(), File::Spec->updir,
+                                               "perl-$vdirs{'5.8.x'}" ),
+                 ftphost => 'public.activestate.com',
+                 ftpusr  => 'anonymous',
+                 ftppwd  => 'smokers@perl.org',
+                 ftpsdir => '/pub/apc/perl-5.8.x',
+                 ftpcdir => '/pub/apc/perl-5.8.x-diffs',
+
                  text   => 'Perl 5.8 MAINT',
-                 cfg    => ( $^O eq 'MSWin32' 
-                        ? 'w32current.cfg' :'perl58x.cfg' ),
+                 cfg    => ( is_win32 ? 'w32current.cfg'
+                           : is_vms ? 'vmsperl.cfg' : 'perl58x.cfg' ),
                  is56x  => 0 },
-    '5.9.x' => { source => 'ftp.linux.activestate.com::perl-current',
-                 server => 'ftp.funet.fi',
-                 sdir   => '/pub/languages/perl/snap/5.9.x',
-                 sfile  => '',
-                 pdir   => '/pub/staff/gsar/APC/perl-current-diffs',
-                 ddir   => File::Spec->rel2abs( 
-                               File::Spec->catdir( File::Spec->updir,
-                                                   'perl-current' ) ),
+
+    '5.9.x' => { source => 'public.activestate.com::perl-current',
+                 server => 'public.activestate.com',
+                 sdir   => '/pub/apc/perl-current-snap',
+                 sfile  => 'perl-current-latest.tar.gz',
+                 pdir   => '/pub/apc/perl-current-diffs',
+                 ddir   => File::Spec->catdir( cwd(), File::Spec->updir,
+                                               'perl-current' ),
+                 ftphost => 'public.activestate.com',
+                 ftpusr  => 'anonymous',
+                 ftppwd  => 'smokers@perl.org',
+                 ftpsdir => '/pub/apc/perl-current',
+                 ftpcdir => '/pub/apc/perl-current-diffs',
+
                  text   => 'Perl 5.10 to-be',
-                 cfg    => ( $^O eq 'MSWin32' 
-                        ? 'w32current.cfg' :'perlcurrent.cfg' ),
+                 cfg    => ( is_win32 ? 'w32current.cfg'
+                           : is_vms ? 'vmsperl.cfg' : 'perlcurrent.cfg' ),
                  is56x  => 0 },
 );
 my @pversions = sort keys %versions;
@@ -183,8 +212,8 @@ my %opt = (
         msg => "Where would you like the new source-tree?
 \tThis directory is also used as the build directory.",
         alt => [ ],
-        dft => File::Spec->rel2abs( File::Spec->catdir( File::Spec->updir,
-                                                        'perl-current' ) ),
+        dft => File::Spec->catdir( File::Spec->rel2abs( File::Spec->updir ),
+                                   'perl-current' ),
         chk => '.+',
     },
     use_old => {
@@ -197,8 +226,9 @@ my %opt = (
     cfg => {
         msg => 'Which build-configuration file would you like to use?',
         alt => [ ],
-        dft => File::Spec->rel2abs( is_win32
-                                    ? 'w32current.cfg' : 'perlcurrent.cfg' ),
+        dft => File::Spec->rel2abs(
+                ( is_win32 ? 'w32current.cfg'
+                : is_vms ? 'vmsperl.cfg' : 'perlcurrent.cfg' ) ),
     },
     change_cfg => {
         msg => undef, # Set later...
@@ -223,7 +253,7 @@ my %opt = (
     },
     # syncing the source-tree
     want_forest => {
-        msg => "Would you like the 'Nick Clark' master sync trees?
+        msg => "Would you like the 'Nick Clark' master sync trees (forest)?
 \tPlease see 'perldoc $0' for an explanation.",
         alt => [qw( N y )],
         dft => 'n',
@@ -238,8 +268,8 @@ my %opt = (
     forest_hdir => {
         msg => 'Where would you like the intermediate source-tree?',
         alt => [ ],
-        dft => File::Spec->rel2abs( File::Spec->catdir( File::Spec->updir,
-                                                        'perl-inter' ) ),
+        dft => File::Spec->catdir( File::Spec->rel2abs( File::Spec->updir ),
+                                   'perl-inter' ),
         chk => '.+',
     },
     fsync => { 
@@ -260,7 +290,7 @@ my %opt = (
     rsync => {
         msg => 'Which rsync program should be used?',
         alt => [ ],
-        dft => whereis( 'rsync' ),
+        dft => ( whereis( 'rsync' ) || '' ),
     },
     opts => {
         msg => 'Which arguments should be used for rsync?',
@@ -292,7 +322,7 @@ my %opt = (
         msg => "How should the snapshots be extracted?
 Examples:$untarmsg",
         alt => [ ],
-        dft => (get_avail_tar())[0],
+        dft => ( (get_avail_tar())[0] || '' ),
     },
 
     snapext => {
@@ -322,7 +352,7 @@ Examples:$untarmsg",
     unzip => {
         msg => 'How should the patches be unzipped?',
         alt => [ ],
-        dft => whereis( 'gzip' ) . " -cd",
+        dft => ( whereis( 'gzip' ) . " -cd" ),
     },
 
     cleanup => {
@@ -346,10 +376,40 @@ Examples:$untarmsg",
         chk => '.+',
     },
 
+    ftphost => {
+        msg => 'From which server should the source-tree be FTPed',
+        alt => [ ],
+        dft => 'ftp.linux.activestate.com',
+        chk => '.+',
+    },
+
+    ftpusr  => {
+       msg => undef, #no choise
+       alt => [ ],
+       dft => 'anonymous',
+    },
+    ftppwd  => {
+       msg => '',
+       alt => [ ],
+       dft => 'smokers@perl.org',
+    },
+    ftpsdir => {
+       msg => 'Base directory for mirror on FTP server',
+       alt => [ ],
+       dft => '/pub/staff/gsar/APC/perl-current',
+       chk => '.+',
+    },
+    ftpcdir => {
+        msg => 'Base directory for patches on FTP server',
+        alt => [ ],
+        dft => '/pub/staff/gsar/APC/perl-current-diffs',
+        chk => '.+',
+    },
+
     patchbin => {
         msg => undef,
         alt => [ ],
-        dft => find_a_patch(),
+        dft => ( find_a_patch() || '' ),
     },
 
     popts => {
@@ -380,7 +440,12 @@ EOT
 Specify a different make program for "make _test".
 EOT
         alt => [ ],
-        dft => $Config{make},
+        dft => ( $Config{make} ? $Config{make} : 'make' ),
+    },
+    harnessonly => {
+        msg => 'Use harness only (skip TEST)?',
+        alt => [qw( y N )],
+        dft => ( $^O =~ /VMS/i ? 'y' : 'n' ),
     },
 
     # mail stuff
@@ -422,6 +487,15 @@ EOMSG
        nck => '\bperl5-porters@perl.org\b',
     },
 
+    swbcc => {
+        msg => <<EOMSG,
+Specify the switch your mailx uses for Blind Carbon Copy (Bcc:) addresses.
+\tSome versions of mailx use '~b' and not '-b'.
+EOMSG
+        alt => [ ],
+        dft => ( $^O =~ /hpux|dec_osf/ ? '~b' : '-b' ),
+    },
+
     cc => {
        msg => <<EOMSG,
 * THIS FEATURE HAS CHANGED IN 1.19! *
@@ -433,6 +507,15 @@ EOMSG
        nck => '\bperl5-porters@perl.org\b',
     },
 
+    swcc => {
+        msg => <<EOMSG,
+Specify the switch your mailx uses for Carbon Copy (Cc:) addresses.
+\tSome versions of mailx use '~c' and not '-c'.
+EOMSG
+        alt => [ ],
+        dft => ( $^O =~ /hpux|dec_osf/ ? '~c' : '-c' ),
+    },
+
     ccp5p_onfail => {
         msg => <<EOMSG,
 Would you like your failed smoke reports CCed to perl5-porters?
@@ -453,8 +536,8 @@ EOMSG
     },
     defaultenv => {
         msg => "Run the test-suite without \$ENV{PERLIO}?",
-        alt => [qw( N y )],
-        dft => 'n',
+        alt => ( is_win32 ? [qw( n Y )] : [qw( N y )] ),
+        dft => ( is_win32 ? 'y' : 'n' ),
     },
     locale => {
         msg => 'What locale should be used for extra testing ' .
@@ -488,6 +571,13 @@ EOT
         alt => [ ],
         dft => "",
     },
+    # Some ENV stuff
+    perl5lib => {
+        msg => "What value should be used for PERL5LIB in the jcl wrapper?
+\t(Make empty, with single space, to not set it.)",
+        alt => [ ],
+        dft => (exists $ENV{PERL5LIB} ? $ENV{PERL5LIB} : ''),
+    },
     # Schedule stuff
     docron => {
         msg => 'Should the smoke be scheduled?',
@@ -504,7 +594,7 @@ EOT
 
 print <<EOMSG;
 
-Welcome to the Perl core smoke test suite. 
+Welcome to the Perl core smoke test suite.
 
 You will be asked some questions in order to configure this test suite.
 Please make sure to read the documentation "perldoc configsmoke.pl"
@@ -543,11 +633,11 @@ F<smokecurrent.sh>. Again the default prefix can be overridden by specifying
 the C<< -j <prefix> >> or C<< -p <prefix> >> switch.
 
 All output (stdout, stderr) from F<smokeperl.pl> and its sub-processes
-is redirected to a logfile called F<smokecurrent.log> by the small jcl. 
+is redirected to a logfile called F<smokecurrent.log> by the small jcl.
 (Use C<< -l <prefix> >> or C<< -p <prefix> >> to override).
 
 There are two additional configuration default files
-F<smoke562_dfconfig> and F<smoke58x_dfconfig> to help you configure 
+F<smoke562_dfconfig> and F<smoke58x_dfconfig> to help you configure
 B<Test::Smoke> for these two maintenance branches of the source-tree.
 
 To create a configuration for the perl 5.8.x branch:
@@ -594,10 +684,9 @@ Here is a description of the configuration sections.
 
 =item perl_version
 
-C<perl_version> sets a number of default_values. 
-This makes the F<smoke5?x_dfconfig> files almost obsolete, 
-although they still provide a nice way to set the prefix
-and set the perl_version.
+C<perl_version> sets a number of default_values.  This makes the
+F<smoke5?x_dfconfig> files almost obsolete, although they still
+provide a nice way to set the prefix and set the perl_version.
 
 =cut
 
@@ -625,10 +714,13 @@ you will need to confirm your choice.
 =cut
 
 {
-    # Hack -des to keep the safeguards
+    # Hack -des (--usedft) to keep the safeguards
     local $options{usedft} = $options{usedft};
     BUILDDIR: {
         $arg = 'ddir';
+        my $ddir = chk_dir( $conf->{$arg} || $config{$arg} )
+                || $opt{$arg}->{dft};
+        $conf->{$arg} = $config{$arg} = $ddir;
         $config{ $arg } = prompt_dir( $arg );
         $options{usedft} = $options{oldcfg};
         my $cwd = cwd;
@@ -637,12 +729,18 @@ you will need to confirm your choice.
             redo BUILDDIR;
         }
         my $bdir = $config{ $arg } = cwd;
+        if ( is_win32 && Win32::FsType() ne 'NTFS' ) {
+            print "*** WHOA THERE!!! ***\n";
+            print "\tYou are on MSWin32, ";
+            print "but do not use a NTFS filesystem to build perl.\n"
+        }
+
         chdir $cwd or die "Can't chdir($cwd) back: $!\n";
         if ( $cwd eq $bdir ) {
             print "The current directory *cannot* be used for smoke testing\n";
             redo BUILDDIR;
         }
-    
+
         $config{ $arg } = File::Spec->canonpath( $config{ $arg } );
         my $manifest  = File::Spec->catfile( $config{ $arg }, 'MANIFEST' );
         my $dot_patch = File::Spec->catfile( $config{ $arg }, '.patch' );
@@ -675,9 +773,13 @@ There are several build-cfg files provided with the distribution:
 
 =back
 
+=begin unsupported
+
 Note: 5.6.2 on MSWin32 is not yet provided, but commenting out the
 B<-Duselargefiles> section from F<w32current.cfg> should be enough.
 
+=end unsupported
+
 =cut
 
 $arg = 'cfg';
@@ -739,9 +841,9 @@ my $want_forest = prompt_yn( $arg );
 FOREST: {
     last FOREST unless $want_forest;
 
-    $config{mdir} = prompt_dir( 'forest_mdir' );
+    $config{mdir} = prompt_dir( 'forest_mdir', 'mdir' );
 
-    $config{fdir} = prompt_dir( 'forest_hdir' );
+    $config{fdir} = prompt_dir( 'forest_hdir', 'fdir' );
 
     $config{sync_type} = 'forest';
 }
@@ -799,7 +901,7 @@ $arg = $want_forest ? 'fsync' : 'sync_type';
 $config{ $arg } = lc prompt( $arg );
 
 SYNCER: {
-    local $_ = $config{ $arg};
+    local *_; $_ = $config{ $arg};
     /^rsync$/ && do {
         $arg = 'source';
         $config{ $arg } = prompt( $arg );
@@ -813,6 +915,15 @@ SYNCER: {
         last SYNCER;
     };
 
+    /^ftp$/  && do {
+        for $arg (qw( ftphost ftpusr ftpsdir ftpcdir )) {
+            $config{ $arg } = prompt( $arg );
+	}
+        $config{ftppwd} = $conf->{ftppwd} || $opt{ftppwd}{dft};
+
+        last SYNCER;
+    };
+
     /^snapshot$/ && do {
         for $arg ( qw( server sdir sfile ) ) {
             if ( $arg ne 'server' && $config{server} =~ m|^https?://|i ) {
@@ -877,6 +988,8 @@ reversing an already applied patch. The file format is simple:
 
 =item * optionally followed by ';' and options to pass to patch
 
+=item * optionally followed by ';' and a description for the patch
+
 =back
 
 If the file does not exist yet, a skeleton version will be created
@@ -906,9 +1019,13 @@ PATCHER: {
         print PATCHES <<EOMSG;
 # Put one filename of a patch on a line, optional args for patch
 # follow the filename separated by a semi-colon (;) [-p1] is default
-# /path/to/patchfile.patch;-p0 -R
+# optionally followed by another ';' and description (added to patchlevel.h)
+# /path/to/patchfile.patch;-p0 -R;Description for this patch
 # Empty lines and lines starting with '#' are ignored
-# File paths are relative to '$config{ddir}' if not absolute
+# File paths are relative to '$config{ddir}'
+# If your patch requires 'regen_perly' you'll need Bison 2 and
+# uncomment the next line (keep the exclamation-point there):
+#!perly
 EOMSG
         close PATCHES or last PATCHER;
         print "Created skeleton '$config{$arg}'\n";
@@ -939,6 +1056,10 @@ if ( $config{is56x} ) {
     $config{ $arg } = 1;
 } else {
     $config{ $arg } = prompt_yn( $arg );
+    if ( is_win32 && ! $config{ $arg } ) {
+        print "*** WHOA THERE!!! ***\n";
+        print "\tYou should not try to use PERLIIO=stdio on MSWin332!\n";
+    }
 }
 =item locale
 
@@ -995,9 +1116,16 @@ MAIL: {
     $config{ $arg } = prompt( $arg );
 
     MAILER: {
-        local $_ = $config{ 'mail_type' };
+        local $_ = $config{mail_type};
 
-        /^mailx?$/         && do { last MAILER };
+        /^mailx$/          && do {
+            if ( $config{bcc} ) {
+                $arg = 'swbcc';
+                $config{ $arg } = prompt( $arg );
+            }
+            last MAILER;
+        };
+        /^mail$/           && do { last MAILER };
         /^sendmail$/       && do {
             $arg = 'from';
             $config{ $arg } = prompt( $arg );
@@ -1017,6 +1145,12 @@ MAIL: {
 
     $arg = 'cc';
     $config{ $arg } = prompt( $arg );
+
+    if ( $config{mail_type} eq 'mailx' && 
+         ( $config{cc} || $config{ccp5p_onfail} ) ) {
+        $arg = 'swcc';
+        $config{ $arg } = prompt( $arg );
+    }
 }
 
 =item w32args
@@ -1059,6 +1193,7 @@ EO_MSG
         ? "Which make should be used" : undef;
 
     $config{w32make} = prompt( 'w32make' );
+    $config{testmake} = $config{testmake};
 
     $config{w32args} = [ 
         "--win32-cctype" => $config{w32cc},
@@ -1068,6 +1203,28 @@ EO_MSG
     ];
 }
 
+=item vmsmake
+
+Get the make program to use for VMS (MMS or MMK). Start with the one
+this perl was build with.
+
+=cut
+
+VMSMAKE: {
+    is_vms or last VMSMAKE;
+
+    my %vmsmakers = get_avail_vms_make();
+
+    $arg = 'vmsmake';
+    $opt{ $arg } = {
+        msg => "Wich maker should be used?",
+        alt => [ sort keys %vmsmakers ],
+        dft => ( $Config{make} || (sort keys %vmsmakers)[0] ),
+    };
+    $config{ $arg } = prompt( $arg );
+    $config{testmake} = $config{ $arg }
+}
+
 =item make finetuning
 
 Two different config options to accomodate the same thing: 
@@ -1082,11 +1239,21 @@ $arg = 'makeopt';
 $opt{ $arg }->{dft} = '-nologo' if is_win32 && $config{w32make} =~ /nmake/i;
 $config{ $arg } = prompt( $arg );
 
-unless ( is_win32 ) {
+unless ( is_win32 || is_vms ) {
     $arg = 'testmake';
     $config{ $arg } = prompt( $arg ) || 'make';
 }
 
+=item harnessonly
+
+C<harnessonly> indicates that C<make test> is replaced by C<make
+test_harness>.
+
+=cut
+
+$arg = 'harnessonly';
+$config{ $arg } = prompt_yn( $arg );
+
 =item umask
 
 C<umask> will be set in the shell-script that starts the smoke.
@@ -1097,7 +1264,7 @@ C<renice> will add a line in the shell-script that starts the smoke.
 
 =cut
 
-unless ( is_win32 ) {
+unless ( is_win32 || is_vms ) {
     $arg = 'umask';
     $config{ $arg } = prompt( $arg );
 
@@ -1168,6 +1335,36 @@ $arg = 'adir';
 $config{ $arg } = prompt_dir( $arg );
 $config{lfile} = File::Spec->rel2abs( $options{log}, cwd );
 
+=item delay_report
+
+Some filesystems do not support opening an already opened file. This
+makes it hard to scan the logfile for compiler messages. We can delay
+the creation of the report and call F<mailrpt.pl> after
+F<smokeperl.pl>. VMS might benefit.
+
+=cut
+
+$arg = 'delay_report';
+$config{ $arg } = $^O =~ /VMS/;
+
+=item ENV stuff
+
+If you have a value for PERL5LIB set in the config environment, you
+could have it transferred tho the jcl-wrapperscript. Do not bother
+asking if it is not there.
+
+=cut
+
+my $has_perl5lib = exists $ENV{PERL5LIB} && defined $ENV{PERL5LIB} &&
+                   length $ENV{PERL5LIB};
+
+P5LIB: {
+    $has_perl5lib or last P5LIB;
+    print "\nI see you have PERL5LIB set to: '$ENV{PERL5LIB}'";
+    $arg = 'perl5lib';
+    $config{ $arg } = prompt( $arg );
+}
+
 =item schedule stuff
 
 =over 4
@@ -1188,7 +1385,8 @@ as F<at.exe> has not got a way comment-out entries.
 =cut
 
 my( $cron, $has_crond,  $crontime );
-SCHEDULE: { 
+SCHEDULE: {
+    is_vms and last SCHEDULE;
     ( $cron, $has_crond ) = get_avail_scheduler();
 
     last SCHEDULE unless $cron;
@@ -1263,23 +1461,31 @@ SAVEALL: {
     save_config();
     if ( is_win32 ) {
         $jcl = write_bat();
+    } elsif ( is_vms ) {
+        $jcl = write_com();
     } else {
         $jcl = write_sh();
     }
 }
 
-print <<EOMSG;
+WRAPUP: {
+    local $" = "";
+    my $chkbcfg = File::Spec->catfile( $findbin, 'chkbcfg.pl' );
+    print <<EOMSG;
+Finished configuration:
 
-Run the perl core test smoke suite with:
+* Please check "$config{cfg}" for the
+  configurations you want to test:
+@{ [map "    $_" => qx( $^X $chkbcfg $config{cfg} )] }
+
+* Run the perl core test smoke suite with:
 \t$jcl
 
-Please check "$config{cfg}" 
-for the configurations you want to test.
-
-Have the appropriate amount of fun!
+* Have the appropriate amount of fun!
 
                                     The Test::Smoke team.
 EOMSG
+}
 
 =back
 
@@ -1325,7 +1531,8 @@ sub sort_configkeys {
 
         # Sync related
         qw( sync_type fsync rsync opts source tar server sdir sfile
-            patchup pserver pdir unzip patchbin cleanup cdir hdir pfile ),
+            patchup pserver pdir unzip patchbin cleanup cdir hdir pfile
+            ftphost ftpusr ftppwd ftpsdir ftpcdir ),
 
         # OS specific make related
         qw( w32args w32cc w32make ),
@@ -1334,13 +1541,17 @@ sub sort_configkeys {
         qw( force_c_locale locale defaultenv ),
 
         # Report related
-        qw( mail mail_type mserver from to ccp5p_onfail cc bcc ),
+        qw( mail mail_type mserver from to ccp5p_onfail
+            swcc cc swbcc bcc ),
 
         # Archive reports and logfile
         qw( adir lfile ),
 
         # make fine-tuning
-        qw( makeopt testmake ),
+        qw( makeopt testmake harnessonly ),
+
+        # ENV stuff
+        qw( perl5lib delay_report ),
     );
 
     my $i = 0;
@@ -1363,9 +1574,15 @@ C<write_sh()> creates the shell-script.
 sub write_sh {
     my $cwd = cwd();
     my $jcl = "$options{jcl}.sh";
-    my $smokeperl = File::Spec->catfile( $FindBin::Bin, 'smokeperl.pl' );
+    my $smokeperl = File::Spec->catfile( $findbin, 'smokeperl.pl' );
     my $cronline = schedule_entry( File::Spec->catfile( $cwd, $jcl ), 
                                    $cron, $crontime );
+
+    my $p5lib = $config{perl5lib} ? <<EO_P5L : '';
+PERL5LIB=$config{perl5lib}
+export PERL5LIB
+EO_P5L
+
     my $handle_lock = $config{killtime} ? <<EO_CONT : <<EO_DIE;
     # Not sure about this, so I will keep the old behaviour 
     # smokeperl.pl will exit(42) on timeout
@@ -1391,15 +1608,16 @@ EO_DIE
 # $cronline
 @{[ renice( $config{renice} ) ]}
 cd $cwd
-CFGNAME=$options{config}
-LOCKFILE=$options{prefix}.lck
+CFGNAME=\${CFGNAME:-$options{config}}
+LOCKFILE=\${LOCKFILE:-$options{prefix}.lck}
 continue=''
 if test -f "\$LOCKFILE" && test -s "\$LOCKFILE" ; then
 $handle_lock
 fi
 echo "\$CFGNAME" > "\$LOCKFILE"
 
-PATH=$FindBin::Bin:$ENV{PATH}
+$p5lib
+PATH=$findbin:$ENV{PATH}
 export PATH
 umask $config{umask}
 $^X $smokeperl -c "\$CFGNAME" \$continue \$\* > $options{log} 2>&1
@@ -1423,22 +1641,32 @@ because it uses commands that are not supported by B<COMMAND.COM>
 
 sub write_bat {
     my $cwd = File::Spec->canonpath( cwd() );
-    my $findbin_bin = File::Spec->canonpath( $FindBin::Bin );
+    my $findbin_bin = File::Spec->canonpath( $findbin );
 
     my $smokeperl  = File::Spec->catfile( $findbin_bin, 'smokeperl.pl' );
     my $archiverpt = File::Spec->catfile( $findbin_bin, 'archiverpt.pl' );
-    my $copycmd = $config{w32args}->[1] ne "BORLAND" ? "" : <<EOCOPYCMD;
+    my $mailrpt    = File::Spec->catfile( $findbin_bin, 'mailrpt.pl' );
+    my $copycmd = <<'EOCOPYCMD';
 
-REM I found hanging XCOPY while smoking with BORLAND
-set COPYCMD=/Y %COPYCMD%
+REM I found hanging XCOPY while smoking; uncommenting the next line might help
+REM set COPYCMD=/Y \%COPYCMD\%
 
 EOCOPYCMD
+    my $p5lib = $config{perl5lib} ? <<EO_P5L : '';
+
+set PERL5LIB=$config{perl5lib}
+EO_P5L
+
 
     my $jcl = "$options{jcl}.cmd";
     my $atline = schedule_entry( File::Spec->catfile( $cwd, $jcl ), 
                                  $cron, $crontime );
-    my $archive = $config{lfile} 
-        ? qq/$^X $archiverpt -c "\%CFGNAME\%"/ : "";
+
+    my $archive = qq/$^X $archiverpt -c "\%CFGNAME\%"/;
+    $config{lfile} or $archive =~ s/^/REM /;
+
+    my $report = qq/$^X $mailrpt -c "\%CFGNAME\%"/;
+    $config{delay_report} or $report =~ s/^/REM /;
 
     local *MYSMOKEBAT;
     open MYSMOKEBAT, "> $jcl" or
@@ -1452,14 +1680,15 @@ REM @{[ scalar localtime ]}
 REM NOTE: Changes made in this file will be \*lost\*
 REM       after rerunning $0
 $copycmd
+$p5lib
 REM $atline
 
 set WD=$cwd\\
 rem Change drive-Letter
 for \%\%L in ( "\%WD\%" ) do \%\%~dL
 cd "\%WD\%"
-set CFGNAME=$options{config}
-set LOCKFILE=$options{prefix}.lck
+if "\%CFGNAME\%"  == "" set CFGNAME=$options{config}
+if "\%LOCKFILE\%" == "" set LOCKFILE=$options{prefix}.lck
 if NOT EXIST \%LOCKFILE\% goto START_SMOKE
     FIND "\%CFGNAME\%" \%LOCKFILE\% > NUL:
     if ERRORLEVEL 1 goto START_SMOKE
@@ -1472,6 +1701,7 @@ if NOT EXIST \%LOCKFILE\% goto START_SMOKE
     set PATH=$findbin_bin;\%PATH\%
     $^X $smokeperl -c "\%CFGNAME\%" \%* > "\%WD\%\\$options{log}" 2>&1
     $archive
+    $report
     set PATH=\%OLD_PATH\%
 
 del \%LOCKFILE\%
@@ -1483,11 +1713,49 @@ EO_BAT
     return File::Spec->canonpath( File::Spec->rel2abs( $jcl ) );
 }
 
+=item write_com
+
+Write a simple DCL script that helps running the smoke suite.
+
+=cut
+
+sub write_com {
+    my $jcl = "$options{jcl}.com";
+    my $cwd = File::Spec->canonpath( cwd() );
+    my $p5lib_p = $config{perl5lib} ? ' '  : '!';
+    my $p5lib = "$p5lib_p DEFINE PERL5LIB $config{perl5lib}";
+    local *MYSMOKECOM;
+    open MYSMOKECOM, "> $jcl" or
+        die "Cannot write '$jcl': $!";
+    print MYSMOKECOM <<EO_COM;
+\$! $jcl - Run Perl5 core test smoke suite
+\$!
+\$! Written by $0 v$VERSION
+\$! @{[ scalar localtime ]}
+\$! NOTE: Changes made in this file will be \*lost\*
+\$!       after rerunning $0
+\$! Try:
+\$!       SUBMIT/NOPRINTER/NOTIFY $cwd$jcl
+\$!
+\$  SET DEFAULT $cwd
+\$$p5lib
+\$! DEFINE/USER sys\$output $options{log}
+\$! DEFINE/USER sys\$error $options{log}
+\$  MCR $^X ${findbin}smokeperl.pl "-c=$options{config}"
+EO_COM
+    close MYSMOKECOM or warn "Error writing '$jcl': $!";
+
+    print "Finished writing '$jcl'\n";
+
+    return '@' . File::Spec->canonpath( File::Spec->rel2abs( $jcl ) );
+}
+
 sub prompt {
     my( $message, $alt, $df_val, $chk, $nck ) = 
         @{ $opt{ $_[0] } }{qw( msg alt dft chk nck )};
 
     $df_val = $conf->{ $_[0] } if exists $conf->{ $_[0] };
+    $df_val = $conf->{ $_[1] } if $_[1] && exists $conf->{ $_[1] };
     unless ( defined $message ) {
         my $retval = defined $df_val ? $df_val : "undef";
         (caller 1)[3] or print "Got [$retval]\n";
@@ -1541,6 +1809,23 @@ sub prompt {
     return $retval;
 }
 
+sub chk_dir {
+    my( $dir ) = @_;
+    defined $dir or return;
+    my $cwd = cwd();
+    File::Path::mkpath( $dir, 1, 0755 ) unless -d $dir;
+ 
+    if ( ! chdir $dir  ) {
+        warn "Cannot chdir($dir): $!\n";
+        $dir = undef;
+    } else {
+        $dir = File::Spec->canonpath( cwd() );
+    }
+    chdir $cwd or die "Cannot chdir($cwd) back: $!";
+
+    return $dir;
+}
+
 sub prompt_dir {
 
     if ( exists $conf->{ $_[0] } && $conf->{ $_[0] } )  {
@@ -1564,11 +1849,7 @@ sub prompt_dir {
                        ( $ENV{HOME} || $ENV{LOGDIR} || 
                          "$ENV{HOMEDRIVE}$ENV{HOMEPATH}" )}ex;
 
-        my $cwd = cwd();
-        File::Path::mkpath( $dir, 1, 0755 ) unless -d $dir;
-        chdir $dir or warn "Cannot chdir($dir): $!\n" and redo GETDIR;
-        $dir = File::Spec->canonpath( cwd() );
-        chdir $cwd or die "Cannot chdir($cwd) back: $!";
+        defined( $dir = chk_dir( $dir ) ) or redo GETDIR;
 
         print "Got [$dir]\n";
         return $dir;
@@ -1613,28 +1894,6 @@ sub prompt_yn {
     return $retval;
 }
 
-sub whereis {
-    my( $prog, $find_all ) = @_;
-    return '' unless $prog; # you shouldn't call it '0'!
-
-    my $p_sep = $Config::Config{path_sep};
-    my @path = split /\Q$p_sep\E/, $ENV{PATH};
-    my @pext = split /\Q$p_sep\E/, $ENV{PATHEXT} || '';
-    unshift @pext, '';
-
-    my @fnames;
-    foreach my $dir ( @path ) {
-        foreach my $ext ( @pext ) {
-            my $fname = File::Spec->catfile( $dir, "$prog$ext" );
-            if ( -x $fname ) {
-                return $fname unless $find_all;
-                push @fnames, $fname;
-            }
-        }
-    }
-    return @fnames ? wantarray ? @fnames : \@fnames : '';
-}
-
 sub find_a_patch {
 
     my $patch_bin;
@@ -1672,6 +1931,9 @@ sub get_avail_sync {
     # (has_ftp && 5.9.x) || has_lwp
     unshift @synctype, 'snapshot' 
         if ( $has_ftp && $pversion eq '5.9.x' ) || $has_lwp;
+
+    unshift @synctype, 'ftp' if $has_ftp;
+
     unshift @synctype, 'rsync' if whereis( 'rsync' );
     return @synctype;
 }
@@ -1681,7 +1943,7 @@ sub get_avail_tar {
     my $use_modules = 0;
     eval { require Archive::Tar };
     unless ( $@ ) {
-        if ( $Archive::Tar::VERSION >= 0.99 ) {
+        if ( eval "$Archive::Tar::VERSION" >= 0.99 ) {
             eval { require IO::Zlib };
         } else {
             eval { require Compress::Zlib };
@@ -1790,8 +2052,7 @@ sub get_avail_w32compilers {
     if ( $map{ $CC }->{ccbin} = whereis( $map{ $CC }->{ccname} ) ) {
         # No, bcc32 doesn't support --version (One can but try)
         my $output = `"$map{ $CC }->{ccbin}" --version 2>&1`;
-        my $ccvers = $output =~ /(\d+.*)/ ? $1 : '?';
-        $ccvers =~ s/\s+copyright.*//i;
+        my $ccvers = $output =~ /([\d.]+)/ ? $1 : '?';
         $map{ $CC }->{ccversarg} = "ccversion=$ccvers";
         $map{ $CC }->{CCTYPE} = 'BORLAND';
     }
@@ -1813,6 +2074,24 @@ sub get_avail_w32compilers {
     } grep length $map{ $_ }->{ccbin} => keys %map;
 }
 
+sub get_avail_vms_make {
+
+    return map +( $_ => undef ) => grep defined $_ && length( $_ ) 
+        => map whereis( $_ ) => qw( MMK MMS );
+ 
+    local *QXERR; open *QXERR, ">&STDERR"; close STDERR;
+
+    my %makers = map {
+        my $maker = $_;
+        map +( $maker => /V([\d.-]+)/ ? $1 : '' )
+        => grep /\b$maker\b/ && /V[\d.-]+/ => qx($maker/IDENT)
+    } qw( MMK MMS );
+
+    open STDERR, ">&QXERR"; close QXERR;
+
+    return %makers;
+}
+
 sub get_Win_version {
     require Test::Smoke::SysInfo;
     ( my $win_version = Test::Smoke::SysInfo::__get_os() ) =~ s/^[^-]*- //;
@@ -1832,10 +2111,11 @@ sub default_buildcfg {
 
     $pversion =~ tr/.//d;
     $pversion eq '59x' and $pversion = 'current';
-    my $basename = $^O eq 'MSWin32'
-        ? "w32current.cfg" : "perl${pversion}.cfg";
+    my $basename = is_win32
+        ? "w32current.cfg"
+        : is_vms ? "vmsperl.cfg" : "perl${pversion}.cfg";
 
-    my $dftbcfg = File::Spec->catfile( $FindBin::Bin, $basename );
+    my $dftbcfg = File::Spec->catfile( $findbin, $basename );
     -f $dftbcfg 
         or die "You seem to have an incomplete Test::Smoke installation" . 
                "($dftbcfg is missing)!\n";
@@ -1865,30 +2145,45 @@ sub check_buildcfg {
 
     my ($rev, @vparts ) = $config{perl_version} =~ /^(\d)(?:\.(\d+))+/;
     my $pversion = sprintf "%d.%03d%02d", $rev, @vparts, 0;
-    print "Checking '$file_name' ($pversion/$^O)\n";
+
+    my $uname_s = Test::Smoke::SysInfo::tsuname( 's' );
+    my( $os, $osver ) = split /\s+-\s+/, $uname_s;
+    # May assume much too much about OS version number formats.
+    my( $osvermaj, $osvermin ) = ($osver =~ /^.+?(\d+)\D+(\d+)/);
+    $osver = sprintf "%s", $osvermaj || '?'; 
+    defined $osvermin and $osver .= sprintf ".%03d", $osvermin;
+
+
+    print "Checking '$file_name'\n     for $pversion on $uname_s\n";
+
     my @no_option = $pversion >= 5.009 ? ( '-Uuseperlio' ) : ( );
     OSCHECK: {
-        local $_ = $^O;
-        /darwin|bsd/i && do { 
+        $os =~ /darwin/ && $osver >= 8 and do {
+            push @no_option, qw( -Duselongdouble -Dusemorebits );
+            last OSCHECK;
+        };
+
+        $os =~ /darwin|bsd/i && do { 
             push @no_option, qw( -Duselongdouble -Dusemorebits -Duse64bitall );
         };
 
-	/linux/i && do {
+	$os =~ /linux/i && do {
             push @no_option, qw( -Duse64bitall );
         };
 
-        /mswin32/i && do {
+        $os =~ /mswin32/i && do {
             push @no_option, qw( -Duselargefiles ) if $config{is56x};
         };
 
-        /cygwin/i && do {
+        $os =~ /cygwin/i && do {
             push @no_option, qw( -Duse64bitall -Duselongdouble -Dusemorebits );
         };
-
-        foreach my $option ( @no_option ) {
-            !/^#/ && /\Q$option\E/ && s/^/#/ for @bcfg;
-        }
     }
+
+    foreach my $option ( @no_option ) {
+        !/^#/ && /\Q$option\E/ && s/^/#/ for @bcfg;
+    }
+
     my $newcfg = join "", grep !/^#/ => @bcfg;
     return if $oldcfg eq $newcfg;
 
@@ -1922,11 +2217,11 @@ sub finish_cfgcheck {
     my $msg = "";
     if ( $overwrite ) {
         my $backup = "$fname.bak";
-        -f $backup and chmod( 0775, $backup ) and unlink $backup;
+        -f $backup and chmod( 0775, $backup ) and 1 while unlink $backup;
         rename $fname, $backup or 
             warn "Cannot rename '$fname' to '$backup': $!";
     } else {
-        $fname = "$options{prefix}.cfg";
+        $fname = "$options{prefix}.cfg.save";
         $msg = " (in case you want it later)";
     }
     # change the filemode (make install used to make perlcurrent.cfg readonly)
@@ -1953,7 +2248,7 @@ Schedule, logfile optional
 
 In case I forget to update the C<$VERSION>:
 
-    $Id: configsmoke.pl 705 2004-05-04 20:08:07Z abeltje $
+    $Id: configsmoke.pl 1032 2007-04-01 20:09:02Z abeltje $
 
 =head1 COPYRIGHT
 
