@@ -1,9 +1,9 @@
 package Test::Smoke;
 use strict;
 
-# $Id: Smoke.pm 716 2004-07-24 17:51:09Z abeltje $
+# $Id: Smoke.pm 1055 2007-08-11 11:32:32Z abeltje $
 use vars qw( $VERSION $REVISION $conf @EXPORT );
-$VERSION  = '1.19';
+$VERSION  = '1.20';
 $REVISION = __get_ts_patchlevel();
 
 use base 'Exporter';
@@ -15,7 +15,8 @@ use Test::Smoke::Policy;
 use Test::Smoke::BuildCFG;
 use Test::Smoke::Smoker;
 use Test::Smoke::SourceTree qw( :mani_const );
-use Test::Smoke::Util qw( get_patch skip_config );
+use Test::Smoke::Util qw( get_patch skip_config
+                          get_local_patches set_local_patch );
 use Config;
 
 =head1 NAME
@@ -29,9 +30,8 @@ Test::Smoke - The Perl core test smoke suite
     use vars qw( $VERSION );
     $VERSION = Test::Smoke->VERSION;
 
-    read_config( $config_name ) or die Test::Smoke->config_error; 
-
-    run_smoke;
+    read_config( $config_name ) or warn Test::Smoke->config_error; 
+    
 
 =head1 DESCRIPTION
 
@@ -98,6 +98,20 @@ sub do_manifest_check {
     }
 }
 
+=item set_smoke_patchlevel( $ddir, $patch[, $verbose] )
+
+Set the current patchlevel as a registered patch like "SMOKE$patch"
+
+=cut
+
+sub set_smoke_patchlevel {
+    my( $ddir, $patch, $verbose ) = @_;
+    $ddir && $patch or return;
+
+    my @smokereg = grep /^SMOKE\d+$/ => get_local_patches( $ddir, $verbose );
+    @smokereg or set_local_patch( $ddir, "SMOKE$patch" );
+}
+
 =item run_smoke( [$continue[, @df_buildopts]] )
 
 C<run_smoke()> sets up de build environment and gets the private Policy
@@ -121,11 +135,6 @@ sub run_smoke {
 
     my $patch = Test::Smoke::Util::get_patch( $conf->{ddir} );
 
-    exists $Config{ldlibpthname} && $Config{ldlibpthname} and
-        $ENV{ $Config{ldlibpthname} } ||= '',
-        substr( $ENV{ $Config{ldlibpthname} }, 0, 0)  =
-            "$conf->{ddir}$Config{path_sep}";
-
     my $logfile = File::Spec->catfile( $conf->{ddir}, 'mktest.out' );
     my $BuildCFG = $continue 
         ? Test::Smoke::BuildCFG->continue( $logfile, $conf->{cfg}, 
@@ -145,13 +154,17 @@ sub run_smoke {
     $conf->{v} && $conf->{defaultenv} and
         $smoker->tty( "Running smoke tests without \$ENV{PERLIO}\n" );
 
-    unless ( $continue ) {
-        $smoker->_make( "-i distclean 2>/dev/null" );
-        $smoker->ttylog( "Smoking patch $patch\n" ); 
-        do_manifest_check( $conf->{ddir}, $smoker );
-    }
+    $conf->{v} && $conf->{harnessonly} and
+        $smoker->tty( "Running smoke tests only with harness.\n" );
 
     chdir $conf->{ddir} or die "Cannot chdir($conf->{ddir}): $!";
+    unless ( $continue ) {
+        $smoker->make_distclean( );
+        $smoker->ttylog( "Smoking patch $patch\n" ); 
+        do_manifest_check( $conf->{ddir}, $smoker );
+        set_smoke_patchlevel( $conf->{ddir}, $patch );
+    }
+
     foreach my $this_cfg ( $BuildCFG->configurations ) {
         $smoker->mark_out; $smoker->mark_in;
         if ( skip_config( $this_cfg ) ) {
@@ -175,8 +188,7 @@ sub run_smoke {
 
 =item __get_ts_patchlevel( )
 
-Read the contents of F<.patch> or use the subversion placeholder
-C<$Rev: 716 $>.
+Read the contents of F<.patch>.
 
 =cut
 
@@ -184,7 +196,7 @@ use FindBin;
 use File::Spec::Functions;
 
 sub __get_ts_patchlevel {
-    my( $rev ) = q$Rev: 716 $ =~ /(\d+)/;
+    my( $rev ) = q$Rev: 1055 $ =~ /(\d+)/;
     my $dotpatch = catfile $FindBin::Bin, '.patch';
     local *DOTPATCH;
     open DOTPATCH, "< $dotpatch" or return $rev;
@@ -199,11 +211,13 @@ sub __get_ts_patchlevel {
 
 =head1 REVISION
 
-$Id: Smoke.pm 716 2004-07-24 17:51:09Z abeltje $
+$Id: Smoke.pm 1055 2007-08-11 11:32:32Z abeltje $
 
 =head1 COPYRIGHT
 
-(c) 2003-2004, Abe Timmerman <abeltje@cpan.org> All rights reserved.
+(c) 2003, All rights reserved.
+
+  * Abe Timmerman <abeltje@cpan.org>
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
