@@ -1,14 +1,15 @@
 package Test::Smoke::Smoker;
 use strict;
 
-# $Id: Smoker.pm 702 2004-05-02 22:47:59Z abeltje $
+# $Id: Smoker.pm 1039 2007-04-02 21:40:38Z abeltje $
 use vars qw( $VERSION );
-$VERSION = '0.015';
+$VERSION = '0.032';
 
 use Cwd;
 use File::Spec::Functions qw( :DEFAULT abs2rel rel2abs );
 use Config;
 use Test::Smoke::Util qw( get_smoked_Config skip_filter );
+require Carp;
 BEGIN { eval q{ use Time::HiRes qw( time ) } }
 
 my %CONFIG = (
@@ -20,6 +21,11 @@ my %CONFIG = (
     df_locale         => '',
     df_force_c_locale => 0,
     df_defaultenv     => 0,
+    df_harness_destruct => 2,
+
+    df_is_vms         => $^O eq 'VMS',
+    df_vmsmake        => 'MMK',
+    df_harnessonly    => scalar ($^O =~ /VMS/),
 
     df_is_win32       => $^O eq 'MSWin32',
     df_w32cc          => 'MSVC60',
@@ -27,7 +33,7 @@ my %CONFIG = (
     df_w32args        => [ ],
 
     df_makeopt        => "",
-    df_testmake       => 'make',
+    df_testmake       => undef,
 );
 
 # Define some constants that we can use for
@@ -36,6 +42,11 @@ sub BUILD_MINIPERL() { -1 } # but no perl
 sub BUILD_PERL    () {  1 } # ok
 sub BUILD_NOTHING () {  0 } # not ok
 
+sub HARNESS_RE1 () {
+     '(\S+\.t)(?:\s+[\d?]+){0,4}(?:\s+[\d?.]*%)?\s+([\d?]+(?:[-\s]+\d+-?)*)$'
+}
+sub HARNESS_RE2() { '^\s+(\d+(?:[-\s]+\d+)*-?)$' }
+
 =head1 NAME
 
 Test::Smoke::Smoker - OO interface to do one smoke cycle.
@@ -85,8 +96,7 @@ sub new {
     my $fh = shift;
 
     unless ( ref $fh eq 'GLOB' ) {
-        require Carp;
-        Carp::croak sprintf "Usage: %s->new( \\*FH, %%args )", __PACKAGE__;
+        Carp::croak( sprintf "Usage: %s->new( \\*FH, %%args )", __PACKAGE__);
     }
 
     my %args_raw = @_ ? UNIVERSAL::isa( $_[0], 'HASH' ) ? %{ $_[0] } : @_ : ();
@@ -104,8 +114,16 @@ sub new {
     $fields{logfh}  = $fh;
     select( ( select( $fh ), $|++ )[0] );
     $fields{defaultenv} = 1 if $fields{is56x};
+    $^O =~ /VMS/i and $fields{is_vms} = 1;
 
-    bless { %fields }, $class;
+    unless ( defined $fields{testmake} ) {
+        $fields{testmake} = 'make';
+        $fields{is_win32} and $fields{testmake} = $fields{w32make};
+        $fields{is_vms}   and $fields{testmake} = $fields{vmsmake};
+    }
+    my $self = bless { %fields }, $class;
+
+    return $self;
 }
 
 sub mark_in {
@@ -120,7 +138,7 @@ sub mark_out {
 
 =item Test::Smoke::Smoker->config( $key[, $value] )
 
-C<config()> is an interface to the package lexical C<%CONFIG>, 
+C<config()> is an interface to the package lexical C<%CONFIG>,
 which holds all the default values for the C<new()> arguments.
 
 With the special key B<all_defaults> this returns a reference
@@ -186,7 +204,7 @@ sub ttylog {
 
 =item $smoker->smoke( $config[, $policy] )
 
-C<smoke()> takes a B<Test::Smoke::BuildCFG::Config> object and runs all 
+C<smoke()> takes a B<Test::Smoke::BuildCFG::Config> object and runs all
 the basic steps as (private) object methods.
 
 =cut
@@ -194,6 +212,8 @@ the basic steps as (private) object methods.
 sub smoke {
     my( $self, $config, $policy ) = @_;
 
+    $self->{is_vms} and $self->_vms_rooted_logical;
+
     $self->make_distclean;
 
     $self->{v} > 1 and $self->extra_manicheck;
@@ -216,14 +236,21 @@ sub smoke {
         return 0;
     };
 
-    my $build_stat = $self->make_;
-  
+    my %sconf = get_smoked_Config( $self->{ddir} => 'ldlibpthname' );
+    exists $sconf{ldlibpthname} or $sconf{ldlibpthname} = "";
+    $sconf{ldlibpthname} and
+        local $ENV{ $sconf{ldlibpthname} } = $ENV{ $sconf{ldlibpthname} } || '',
+        substr( $ENV{ $sconf{ldlibpthname} }, 0, 0) =
+            "$self->{ddir}$Config{path_sep}";
+
+    my $build_stat = $self->make_( $config );
+
     $build_stat == BUILD_MINIPERL and do {
         $self->ttylog( "Unable to make anything but miniperl",
                        " in this configuration\n" );
-        return $self->make_minitest( "$config" );
+        return $self->make_minitest;
     };
-       
+
     $build_stat == BUILD_NOTHING and do {
         $self->ttylog( "Unable to make perl in this configuration\n" );
         return 0;
@@ -234,8 +261,9 @@ sub smoke {
         return 0;
     };
 
-    $self->make_test( "$config" );
+    $self->make_test( $config );
 
+#    $self->{is_vms} and $self->_unset_rooted_logical;
     return 1;
 }
 
@@ -247,7 +275,7 @@ C<make_distclean()> runs C<< make -i distclean 2>/dev/null >>
 
 sub make_distclean {
     my $self = shift;
-    
+
     $self->tty( "make distclean ..." );
     if ( $self->{fdir} && -d $self->{fdir} ) {
         require Test::Smoke::Syncer;
@@ -259,7 +287,8 @@ sub make_distclean {
         my $distclean = Test::Smoke::Syncer->new( hardlink => %options );
         $distclean->clean_from_directory( $self->{fdir}, 'mktest.out' );
     } else {
-        $self->_make( "-i distclean 2>/dev/null" );
+        my $target = $self->{is_vms} ? 'realclean' : '-i distclean';
+        $self->_make( "$target 2>/dev/null" );
     }
 }
 
@@ -290,7 +319,7 @@ sub extra_manicheck {
 
 =item $smoker->handle_policy( $policy, @substs );
 
-C<handle_policy()> will try to apply the substition rules and then 
+C<handle_policy()> will try to apply the substition rules and then
 write the file F<Policy.sh>.
 
 =cut
@@ -326,9 +355,16 @@ sub Configure {
     if ( $self->{is_win32} ) {
         my @w32args = @{ $self->{w32args} };
         @w32args = @w32args[ 4 .. $#w32args ];
-        $makefile = $self->_run( "./Configure $config", 
+        my $w32_cfg = "$config" =~ /-DCCTYPE=/
+            ? "$config" : "$config -DCCTYPE=$self->{w32cc}";
+
+        $makefile = $self->_run( "./Configure $w32_cfg",
                                  \&Test::Smoke::Util::Configure_win32,
                                  $self->{w32make}, @w32args  );
+    } elsif ( $self->{is_vms} ) {
+        my $vms_cfg = $config->vms;
+        $self->_run( qq/\@configure -"des" $vms_cfg/ );
+        $makefile = 'DESCRIP.MMS';
     } else {
         $self->_run( "./Configure -des $config" );
         $makefile = 'Makefile';
@@ -346,9 +382,18 @@ returns true if a perl executable is found
 
 sub make_ {
     my $self = shift;
+    my $config = shift;
 
     $self->tty( "\nmake ..." );
-    $self->_make( "" );
+    my $make_args = "";
+    $self->{is_vms} && $config->has_arg( '-Dusevmsdebug' ) and
+        $make_args = qq[/macro=("__DEBUG__=1")];
+
+    $self->{is_win32} && $config->has_arg( '-Uuseshrplib' ) and
+        $make_args = "static";
+
+    my $make_output = $self->_make( $make_args );
+
     if ( $self->{is_win32} ) { # Win32 creates config.sh during make
         my %cinfo = get_smoked_Config( $self->{ddir} => qw(
             cc ccversion gccversion
@@ -356,13 +401,23 @@ sub make_ {
         my $version = $cinfo{gccversion} || $cinfo{ccversion};
         $self->log( "\nCompiler info: $cinfo{cc} version $version\n" )
             if $cinfo{cc};
+
+        $self->{w32cc} =~ /MSVC|BORLAND/ and $self->tty( "\n$make_output\n" );
     }
 
     my $exe_ext  = $Config{_exe} || $Config{exe_ext};
     my $miniperl = "miniperl$exe_ext";
     my $perl     = "perl$exe_ext";
+    $perl = "ndbg$perl"
+        if $self->{is_vms} && $config->has_arg( '-Dusevmsdebug' );
+    $perl = "perl-static$exe_ext"
+        if $self->{is_win32} && $config->has_arg( '-Uuseshrplib' );
+
+    $self->{_miniperl_bin} = $miniperl;
+    $self->{_perl_bin}     = $perl;
+
     -x $miniperl or return BUILD_NOTHING;
-    return -x $perl 
+    return -x $perl
         ? $self->{_run_exit} ? BUILD_MINIPERL : BUILD_PERL
         : BUILD_MINIPERL;
 }
@@ -375,9 +430,9 @@ Run C<< I<make test-perp> >> and check if F<t/perl> exists.
 
 sub make_test_prep {
     my $self = shift;
+    $self->{harnessonly} and return 1; # no test-prep target
 
-    my $exe_ext = $Config{_exe} || $Config{exe_ext};
-    my $perl = catfile( "t", "perl$exe_ext" );
+    my $perl = catfile( "t", $self->{_perl_bin} );
 
     $self->{run} and unlink $perl;
     $self->_make( "test-prep" );
@@ -391,7 +446,8 @@ sub make_test_prep {
 
 sub make_test {
     my $self = shift;
-    my( $config_args ) = @_;
+    my( $config ) = @_;
+    my $config_args = "$config";
 
     $self->tty( "\n Tests start here:\n" );
 
@@ -400,7 +456,7 @@ sub make_test {
     my @layers = ( ($config_args =~ /-Uuseperlio\b/) || $self->{defaultenv} )
                ? qw( stdio ) : qw( stdio perlio );
 
-    if ( !($config_args =~ /-Uuseperlio\b/ || $self->{defaultenv}) && 
+    if ( !($config_args =~ /-Uuseperlio\b/ || $self->{defaultenv}) &&
          $self->{locale} ) {
         push @layers, 'locale';
     }
@@ -430,54 +486,24 @@ sub make_test {
             next;
 	}
 
-        my $test_target = $self->{is56x} ? 'test-notty' : '_test';
-        local *TST;
-        # MSWin32 builds from its own directory
-        if ( $self->{is_win32} ) {
-            chdir "win32" or die "unable to chdir () into 'win32'";
-            # Same as in make ()
-            open TST, "$self->{w32make} -f smoke.mk $test_target |";
-            chdir ".." or die "unable to chdir () out of 'win32'";
+	if  ( $self->{harnessonly} ) {
+
+            $self->make_test_harness( $config );
+
         } else {
-            local $ENV{PERL} = "./perl";
-            open TST, "$self->{testmake} $test_target |" or do {
-                use Carp;
-                Carp::carp "Cannot fork 'make _test': $!";
-                next;
-            };
-        }
+            my $test_target = $self->{is_vms}
+                ? 'test' : $self->{is56x} ? 'test-notty' : '_test';
 
-        my @nok = ();
-        select ((select (TST), $| = 1)[0]);
-        while (<TST>) {
-            $self->{v} > 1 and $self->tty( $_ );
-            skip_filter( $_ ) and next;
-
-            # make mkovz.pl's life easier
-            s/(.)(TSTENV\s+=\s+\w+)/$1\n$2/;
-
-            if (m/^u=.*tests=/) {
-                s/(\d\.\d*) /sprintf "%.2f ", $1/ge;
-                $self->log( $_ );
+            # MSWin32 builds from its own directory
+            if ( $self->{is_win32} ) {
+                $config->has_arg( '-Uuseshrplib' )
+                    and $test_target = 'static-test';
+                $self->_run_harness_target( $test_target );
             } else {
-                push @nok, $_;
+                $self->_run_TEST_target( $test_target, 1 );
             }
-            $self->tty( $_ );
+            $self->tty( "\n" );
         }
-        close TST or do {
-            my $error = $! || ( $? >> 8);
-            require Carp;
-            Carp::carp "\nError while reading test-results: $error";
-        };
-#        $self->log( map { "    $_" } @nok );
-        if (grep m/^All tests successful/, @nok) {
-            $self->log( "All tests successful.\n" );
-            $self->tty( "\nOK, archive results ..." );
-            $self->{patch} and $nok[0] =~ s/\./ for .patch = $self->{patch}./;
-        } else {
-            $self->extend_with_harness( @nok );
-        }
-        $self->tty( "\n" );
         !$had_LC_ALL && exists $ENV{LC_ALL} and delete $ENV{LC_ALL};
     }
 
@@ -492,37 +518,36 @@ sub extend_with_harness {
     my $self = shift;
     my %inconsistent = $self->_transform_testnames( @_ );
     my @harness = sort keys %inconsistent;
+    my $harness_re1 = HARNESS_RE1();
+    my $harness_re2 = HARNESS_RE2();
     if ( @harness ) {
+
+        # @20051016 By request of Nicholas Clark
+        local $ENV{PERL_DESTRUCT_LEVEL} = $self->{harness_destruct};
         local $ENV{PERL_SKIP_TTY_TEST} = 1;
+
+        # I'm not happy with this PERLSHR approach for VMS
+        local $ENV{PERLSHR} = $ENV{PERLSHR} || "";
+        $self->{is_vms} and
+             $ENV{PERLSHR} = catfile( $self->{ddir},
+                                      'PERLSHR' . $Config{_exe} );
         my $harness = join " ", @harness;
         $self->tty( "\nExtending failures with harness:\n\t$harness\n" );
         my $changed_dir;
         chdir 't' and $changed_dir = 1;
-        my $harness_all_ok = 0;
+        my $all_ok = 0;
         my $tst_perl = catfile( curdir(), 'perl' );
-        my $harness_out = join "", map {
-            my( $name, $fail ) = 
-                m/(\S+\.t)\s+.+%\s+([\d?]+(?:[-\s]+\d+)*)/;
-            if ( $name ) {
-                delete $inconsistent{ $name };
-                my $dots = '.' x (40 - length $name );
-                "    $name${dots}FAILED $fail\n";
-            } else {
-                ( $fail ) = m/^\s+(\d+(?:[-\s]+\d+)*)/;
-                " " x 51 . "$fail\n";
-            }
-        } grep m/^\s+\d+(?:[-\s]+\d+)*/ ||
-               m/\S+\.t\s+.+%\s+[\d?]+(?:[-\s+]\d+)*/ => map {
-            /All tests successful/ && $harness_all_ok++;
-            $self->{v} > 1 and $self->tty( $_ );
-            $_;
-        } $self->_run( "$tst_perl harness $harness" );
+        my $verbose = $self->{v} > 1 ? "-v" : "";
+        my @run_harness = $self->_run( "$tst_perl harness $verbose $harness" );
+        my $harness_out = $self->_parse_harness( \%inconsistent, $all_ok,
+                                                 @run_harness );
+
         # safeguard against empty results
         $inconsistent{ $_ } ||= 'FAILED' for keys %inconsistent;
         $harness_out =~ s/^\s*$//;
-        if ( $harness_all_ok ) {
+        if ( $all_ok ) {
             $harness_out .= scalar keys %inconsistent
-                ? "Inconsistent test results (between TEST and harness):\n" . 
+                ? "Inconsistent test results (between TEST and harness):\n" .
                   join "", map {
                       my $dots = '.' x (40 - length $_ );
                       "    $_${dots}$inconsistent{ $_ }\n";
@@ -530,7 +555,7 @@ sub extend_with_harness {
                 : $harness_out ? "" : "All tests successful.";
         } else {
             $harness_out .= scalar keys %inconsistent
-                ? "Inconsistent test results (between TEST and harness):\n" . 
+                ? "Inconsistent test results (between TEST and harness):\n" .
                   join "", map {
                       my $dots = '.' x (40 - length $_ );
                       "    $_${dots}$inconsistent{ $_ }\n";
@@ -542,7 +567,123 @@ sub extend_with_harness {
     }
 }
 
-=item $self->make_minitest( $cfgargs )
+=item $moker->make_test_harness
+
+Use Test::Harness (the test_harness target) to get the failing test
+information and do not bother with TEST.
+
+=cut
+
+sub make_test_harness {
+    my( $self, $config ) = @_;
+
+    my $target= "test_harness";
+    my $debugging = "";
+
+    if ( $self->{is_vms} ) {
+
+        $debugging = $config->has_arg( '-Dusevmsdebug' )
+            ? qq[/macro=("__DEBUG__=1")]
+            : "";
+
+    } elsif ($self->{is_win32}) {
+        $target = $config->has_arg( '-Uuseshrplib' ) ? "static-test" : "test";
+    }
+
+    $self->_run_harness_target( $target, $debugging );
+}
+
+=item $smoker->_run_harness_target( $target, $extra )
+
+The command to run C<make test_harness> differs based on platform, so
+the arguments have to be passed into general routine. C<$target>
+specifies the makefile-target, C<$makeopt> specifies the extra options
+for the make program.
+
+=cut
+
+sub _run_harness_target {
+    my( $self, $target, $extra ) = @_;
+
+    my $seenheader = 0;
+    my @failed = ( );
+
+    my $harness_re1 = HARNESS_RE1();
+    my $harness_re2 = HARNESS_RE2();
+
+    my $tst = $self->_make_fork( $target, $extra );
+
+    while ( <$tst> ) {
+        $self->{v} > 1 and $self->tty( $_ );
+
+        /All tests successful/ and push( @failed, $_ ), last;
+
+        /Failed Test\s+Stat/ and $seenheader = 1, next;
+        $seenheader or next;
+
+        my( $name, $fail ) = m/$harness_re1/;
+        if ( $name ) {
+            my $dots = '.' x (40 - length $name );
+            push @failed, "    $name${dots}FAILED $fail\n";
+        } else {
+            ( $fail ) = m/$harness_re2/;
+            next unless $fail;
+            push @failed, " " x 51 . "$fail\n";
+        }
+    }
+
+    close $tst or do {
+        my $error = $! || ( $? >> 8);
+        Carp::carp( "\nerror while running harness target '$target': $error" );
+    };
+
+    $self->ttylog( "\n", join( "", @failed ), "\n" );
+    $self->tty( "Archived results...\n" );
+}
+
+sub _run_TEST_target {
+    my( $self, $target, $extend ) = @_;
+    !$target and Carp::Confess( "No target in _run_TEST_target" );
+
+    my @nok;
+    my $tst = $self->_make_fork( $target );
+    my $ok;
+
+    while (<$tst>) {
+        $self->{v} > 1 and $self->tty( $_ );
+        skip_filter( $_ ) and next;
+
+        # make mkovz.pl's life easier
+        s/(.)(TSTENV\s+=\s+\w+)/$1\n$2/;
+
+        if (m/^u=.*tests=/) {
+            s/(\d\.\d*) /sprintf "%.2f ", $1/ge;
+            $self->log( $_ );
+        } else {
+            $ok ||= m/^All tests successful/;
+            push @nok, $_;
+        }
+        $self->tty( $_ );
+    }
+    close $tst or do {
+        my $error = $! || ( $? >> 8);
+        Carp::carp("\nError while reading test-results: $error");
+    };
+
+#    $self->log( map { "    $_" } @nok );
+    if ( grep m/^All tests successful/, @nok ) {
+        $self->log( "All tests successful.\n" );
+        $self->tty( "\nOK, archive results ..." );
+        $self->{patch} and
+            $nok[0] =~ s/\./ for .patch = $self->{patch}./;
+    } elsif ( !$extend ) {
+        $self->ttylog( map { "    $_" } @nok );
+    } else {
+        $self->extend_with_harness( @nok );
+    }
+}
+
+=item $self->make_minitest
 
 C<make> was unable to build a I<perl> executable, but managed to build
 I<miniperl>, so we do C<< S<make minitest> >>.
@@ -553,49 +694,49 @@ sub make_minitest {
     my $self = shift;
 
     $self->ttylog( "TSTENV = minitest\t" );
-    local *TST;
-    # MSWin32 builds from its own directory
-    if ( $self->{is_win32} ) {
-        chdir "win32" or die "unable to chdir () into 'win32'";
-        # Same as in make ()
-        open TST, "$self->{w32make} -f smoke.mk minitest |";
-        chdir ".." or die "unable to chdir () out of 'win32'";
+
+    if ($self->{is_win32}) {
+        $self->_run_harness_target( "minitest" );
     } else {
-        local $ENV{PERL} = "./perl";
-        open TST, "make minitest |" or do {
-            use Carp;
-            Carp::carp "Cannot fork 'make _test': $!";
-            return 0;
-        };
+        $self->_run_TEST_target( "minitest", 0 );
     }
 
-    my @nok = ();
-    select ((select (TST), $| = 1)[0]);
-    while (<TST>) {
-        $self->{v} >= 2 and $self->tty( $_ );
-        skip_filter( $_ ) and next;
-        # make mkovz.pl's life easier
-        s/(.)(PERLIO\s+=\s+\w+)/$1\n$2/;
-
-        if (m/^u=.*tests=/) {
-            s/(\d\.\d*) /sprintf "%.2f ", $1/ge;
-            $self->log( $_ );
-        } else {
-            push @nok, $_;
-        }
-        $self->tty( $_ );
-    }
-    close TST or do {
-        require Carp;
-        Carp::carp "Error while reading pipe: $!";
-    };
-    $self->ttylog( map { "    $_" } @nok );
-
-    $self->tty( "\nOK, archive results ..." );
     $self->tty( "\n" );
     return 1;
 }
 
+=item $self->_parse_harness_output( $\%notok, $all_ok, @lines )
+
+Fator out the parsing of the Test::Harness output, as it seems subject
+to change.
+
+=cut
+
+sub _parse_harness {
+    my( $self, $notok, $all_ok, @lines ) = @_;
+    my $harness_re1 = HARNESS_RE1();
+    my $harness_re2 = HARNESS_RE2();
+
+    my $output = join "", map {
+        my( $name, $fail ) = m/$harness_re1/;
+        if ( $name ) {
+            delete $notok->{ $name };
+            my $dots = '.' x (40 - length $name );
+            "    $name${dots}FAILED $fail\n";
+        } else {
+            ( $fail ) = m/$harness_re2/;
+            " " x 51 . "$fail\n";
+        }
+    } grep m/$harness_re2/ || m/$harness_re1/ => map {
+        /All tests successful/ && $all_ok++;
+        $self->{v} and $self->tty( $_ );
+        $_;
+    } @lines;
+
+    $_[2] = $all_ok;
+    return $output;
+}
+
 =item $self->_trasnaform_testnames( @notok )
 
 C<_transform_testnames()> takes a list of testnames, as found by
@@ -639,6 +780,7 @@ sub _run {
     my $self = shift;
     my( $command, $sub, @args ) = @_;
 
+    $self->{v} > 1 and print "[$command]\n";
     defined $sub and return &$sub( $command, @args );
 
     my @output = qx( $command );
@@ -658,19 +800,80 @@ sub _make {
     my $cmd = shift;
     $self->{makeopt} and $cmd = "$self->{makeopt} $cmd";
 
-    $self->{is_win32} or return $self->_run( "make $cmd" );
+    $self->{is_win32} || $self->{is_vms} or return $self->_run( "make $cmd" );
 
     my $kill_err;
     # don't capture STDERR
     # @ But why? and what if we do it DOSish? 2>NUL:
 
-    my $win32_maker = $self->{w32make};
+    my $maker = $self->{is_vms} ? $self->{vmsmake} : $self->{w32make};
     $cmd =~ s|2\s*>\s*/dev/null\s*$|| and $kill_err = 1;
 
-    $cmd = "$win32_maker -f smoke.mk $cmd";
-    chdir "win32" or die "unable to chdir () into 'win32'";
-    $self->_run( $kill_err ? qq{$^X -e "close STDERR; system '$cmd'"} : $cmd );
-    chdir ".." or die "unable to chdir() out of 'win32'";
+    if ( $self->{is_win32} ) {
+        $cmd = "$maker -f smoke.mk $cmd";
+        chdir "win32" or die "unable to chdir () into 'win32'";
+    } else {
+        $cmd = "$maker $cmd";
+    }
+    my @output = $self->_run(
+        $kill_err ? qq{$^X -e "close STDERR; system '$cmd'"} : $cmd
+    );
+    if ( $self->{is_win32} ) {
+        chdir ".." or die "unable to chdir() out of 'win32'";
+    }
+    return wantarray ? @output : join "", @output;
+}
+
+=item $smoker->_make_fork( $target, $extra )
+
+C<_make_fork()> opens a read pipe to the make command with C<$target>
+and C<$extra> arguments for the make command.
+
+=cut
+
+sub _make_fork {
+    my( $self, $target, $extra ) = @_;
+
+    !defined $extra and $extra = "";
+
+    my( $ok, $err, $cmd );
+    local *TST;
+
+    # MSWin32 builds from its own directory
+    if ( $self->{is_win32} ) {
+        chdir "win32" or die "unable to chdir () into 'win32'";
+        # Same as in make ()
+        $cmd = "$self->{testmake}$extra -f smoke.mk $target |";
+        $ok = open TST, $cmd  or $err = $!;
+        chdir ".." or die "unable to chdir () out of 'win32'";
+    } else {
+        local $ENV{PERL} = "./perl";
+        $cmd = "$self->{testmake}$extra $target |";
+        $ok = open TST, $cmd or $err = $!;
+    }
+    $ok or do {
+        Carp::carp("Cannot fork '$cmd': $err");
+        return 0;
+    };
+    select ((select (*TST), $| = 1)[0]);
+    return *TST;
+}
+
+=item $smoker->_vms__rooted_logical
+
+This code sets up a rooted logical C<TSP5SRC> and changes the {ddir}
+to that root.
+
+=cut
+
+sub _vms_rooted_logical {
+    my $self = shift;
+    return unless $^O eq 'VMS';
+
+    Test::Smoke::Util::set_vms_rooted_logical( TSP5SRC => $self->{ddir} );
+    $self->{vms_ddir} = $self->{ddir};
+    $self->{ddir} = 'TSP5SRC:[000000]';
+
 }
 
 1;
