@@ -1,44 +1,47 @@
 package Test::Smoke::Syncer;
 use strict;
 
-# $Id: Syncer.pm 322 2003-08-03 15:33:36Z abeltje $
+# $Id: Syncer.pm 1037 2007-04-02 21:17:19Z abeltje $
 use vars qw( $VERSION );
-$VERSION = '0.008';
+$VERSION = '0.022';
 
+use Config;
 use Cwd;
 use File::Spec;
 require File::Path;
 
 my %CONFIG = (
-    df_sync => 'rsync',
-    df_ddir => File::Spec->rel2abs( 'perl-current', File::Spec->curdir ),
-    df_v    => 0,
+    df_sync     => 'rsync',
+    df_ddir     => File::Spec->rel2abs( 'perl-current', File::Spec->curdir ),
+    df_v        => 0,
 
 # these settings have to do synctype==rsync
-    df_rsync  => 'rsync', # you might want a path there
-    df_opts   => '-az --delete',
-    df_source => 'ftp.linux.activestate.com::perl-current',
+    df_rsync    => 'rsync', # you might want a path there
+    df_opts     => '-az --delete',
+    df_source   => 'public.activestate.com::perl-current',
 
-    rsync     => [qw( rsync source opts )],
+    rsync       => [qw( rsync source opts )],
 
 # these settings have to do with synctype==snapshot
-    df_ftp     => 'Net::FTP',
-    df_server  => 'ftp.funet.fi',
-    df_sdir    => '/pub/languages/perl/snap',
-    df_sfile   => '',
-    df_snapext => 'tgz',
+    df_ftp      => 'Net::FTP',
+    df_server   => 'public.activestate.com',
+    df_sdir     => '/pub/apc/perl-current-snap',
+    df_sfile    => '',
+    df_snapext  => 'tar.gz',
 
-    df_tar     => ( $^O eq 'MSWin32' ?
+    df_tar      => ( $^O eq 'MSWin32' ?
         'Archive::Tar' : 'gzip -d -c %s | tar xf -' ),
 
-    df_patchup => 0,
-    df_pserver => 'ftp2.activestate.com',
-    df_pdir    => '/pub/staff/gsar/APC/perl-current-diffs',
-    df_unzip   => $^O eq 'MSWin32' ? 'Compress::Zlib' : 'gzip -dc',
-    df_patch   => 'patch',
-    df_cleanup => 1,
-    snapshot   => [qw( ftp server sdir sfile snapext tar 
-                       patchup pserver pdir unzip patch cleanup )],
+    df_patchup  => 0,
+    df_pserver  => 'public.activestate.com',
+    df_pdir     => '/pub/apc/perl-current-diffs',
+    df_ftpusr   => 'anonymous',
+    df_ftppwd   => 'smokers@perl.org',
+    df_unzip    => $^O eq 'MSWin32' ? 'Compress::Zlib' : 'gzip -dc',
+    df_patchbin => 'patch',
+    df_cleanup  => 1,
+    snapshot    => [qw( ftp server sdir sfile snapext tar ftpusr ftppwd
+                       patchup pserver pdir unzip patchbin cleanup )],
 
 # these settings have to do with synctype==copy
     df_cdir    => undef,
@@ -47,16 +50,27 @@ my %CONFIG = (
 
 # these settings have to do with synctype==hardlink
     df_hdir    => undef,
+    df_haslink => ($Config{d_link}||'') eq 'define',
 
-    hardlink   => [qw( hdir )],
+    hardlink   => [qw( hdir haslink )],
 
+# these have to do 'forest'
     df_fsync   => 'rsync',
     df_mdir    => undef,
     df_fdir    => undef,
+
     forest     => [qw( fsync mdir fdir )],
 
+# these settings have to do with synctype==ftp
+    df_ftphost => 'public.activestate.com',
+    df_ftpsdir => '/pub/apc/perl-current',
+    df_ftpcdir => '/pub/apc/perl-current-diffs',
+
+    ftp        => [qw( ftphost ftpusr ftppwd ftpsdir ftpcdir )],
+
 # misc.
-    valid_type => { rsync => 1, snapshot => 1, copy => 1, hardlink => 1 },
+    valid_type => { rsync => 1, snapshot => 1,
+                    copy  => 1, hardlink => 1, ftp => 1 },
 );
 
 {
@@ -143,7 +157,7 @@ sub new {
 
     unless ( exists $CONFIG{valid_type}->{$sync_type} ) {
         require Carp;
-        Carp::croak "Invalid sync_type '$sync_type'";
+        Carp::croak( "Invalid sync_type '$sync_type'" );
     };
 
     my %args_raw = @_ ? UNIVERSAL::isa( $_[0], 'HASH' ) ? %{ $_[0] } : @_ : ();
@@ -157,19 +171,23 @@ sub new {
         my $value = exists $args{$_} ? $args{ $_ } : $CONFIG{ "df_$_" };
         ( $_ => $value )
     } ( v => ddir => @{ $CONFIG{ $sync_type } } );
+    if ( ! File::Spec->file_name_is_absolute( $fields{ddir} ) ) {
+        $fields{ddir} = File::Spec->catdir( cwd(), $fields{ddir} );
+    }
     $fields{ddir} = File::Spec->rel2abs( $fields{ddir} );
 
     DO_NEW: {
-        local $_ = $sync_type;
+        local *_; $_ = $sync_type;
 
         /^rsync$/    && return Test::Smoke::Syncer::Rsync->new( %fields );
         /^snapshot$/ && return Test::Smoke::Syncer::Snapshot->new( %fields );
         /^copy$/     && return Test::Smoke::Syncer::Copy->new( %fields );
         /^hardlink$/ && return Test::Smoke::Syncer::Hardlink->new( %fields );
+        /^ftp$/      && return Test::Smoke::Syncer::FTP->new( %fields );
         /^forest$/   && return Test::Smoke::Syncer::Forest->new( %fields );
 
         require Carp;
-        Carp::croak "Synctype '$_', not yet implemented.";
+        Carp::croak( "Synctype '$_', not yet implemented." );
     }
 
 }
@@ -247,11 +265,17 @@ sub _relocate_tree {
     $self->{v} and print "relocate source-tree ";
 
     # try to move it at once (sort of a rename)
-    my $ok = $source_dir eq $self->{ddir}
+    my $ddir = $^O eq 'VMS' ? $self->{vms_ddir} : $self->{ddir};
+    my $ok = $source_dir eq $ddir
            ? 1 : File::Copy::move( $source_dir, $self->{ddir} );
 
     # Failing that: Copy-By-File :-(
     if ( ! $ok && -d $source_dir ) {
+        my $cwd = cwd();
+        chdir $source_dir or do {
+            print "Cannot chdir($source_dir): $!\n";
+            return 0;
+        };
         require File::Find;
 	File::Find::finddepth( sub {
 
@@ -261,11 +285,12 @@ sub _relocate_tree {
 
             $self->{v} > 1 and print "move $_ $dest\n";
 	    File::Copy::move( $_, $dest );
-        }, $source_dir );
+        }, "./" );
+        chdir $cwd or print "Cannot chdir($cwd) back: $!\n";
 	File::Path::rmtree( $source_dir, $self->{v} > 1 );
         $ok = ! -d $source_dir;
     }
-    die "Can't move '$source_dir' to $self->{ddir}'" unless $ok;
+    die "Can't move '$source_dir' to $self->{ddir}' ($!)" unless $ok;
     $self->{v} and print "OK\n";
 }
 
@@ -342,21 +367,8 @@ from the F<patchlevel.h> file in the distribution.
 sub version_from_patchlevel_h {
     my $self = shift;
 
-    my $file = File::Spec->catfile( $self->{ddir}, 'patchlevel.h' );
-
-    my( $revision, $version, $subversion ) = qw( 5 ? ? );
-    local *PATCHLEVEL;
-    if ( open PATCHLEVEL, "< $file" ) {
-        my $patchlevel = do { local $/; <PATCHLEVEL> };
-        close PATCHLEVEL;
-        $revision   = $patchlevel =~ /^#define PERL_REVISION\s+(\d+)/m 
-                    ? $1 : '?';
-        $version    = $patchlevel =~ /^#define PERL_VERSION\s+(\d+)/m
-                    ? $1 : '?';
-        $subversion = $patchlevel =~ /^#define PERL_SUBVERSION\s+(\d+)/m 
-                    ? $1 : '?';
-    }
-    return "$revision.$version.$subversion";
+    require Test::Smoke::Util;
+    return Test::Smoke::Util::version_from_patchelevel( $self->{ddir} );
 }
  
 =item $syncer->clean_from_directory( $source_dir[, @leave_these] )
@@ -395,10 +407,46 @@ sub clean_from_directory {
     }, $self->{ddir} );
 }
 
+=item $syncer->pre_sync
+
+C<pre_sync()> should be called by the C<sync()> methos to setup the
+sync environment. Currently only useful on I<OpenVMS>.
+
+=cut
+
+sub pre_sync {
+    return 1 unless $^O eq 'VMS';
+    my $self = shift;
+    require Test::Smoke::Util;
+
+    Test::Smoke::Util::set_vms_rooted_logical( TSP5SRC => $self->{ddir} );
+    $self->{vms_ddir} = $self->{ddir};
+    $self->{ddir} = 'TSP5SRC:[000000]';
+}
+
 # Set skeleton-sub
 sub sync { 
     require Carp; 
-    Carp::croak ref( $_[0] ) . "->sync() not yet implemented.";
+    Carp::croak( ref( $_[0] ) . "->sync() not yet implemented." );
+}
+
+=item $syncer->post_sync
+
+C<post_sync()> should be called by the C<sync()> methos to unset the
+sync environment. Currently only useful on I<OpenVMS>.
+
+=cut
+
+sub post_sync {
+    return 1 unless $^O eq 'VMS';
+    my $self = shift;
+
+    ( my $logical = $self->{ddir} || '' ) =~ s/:\[000000\]$//;
+    return unless $logical;
+    my $result = system "DEASSIGN/JOB $logical";
+
+    $self->{ddir} = delete $self->{vms_ddir};
+    return $result == 0;
 }
 
 1;
@@ -451,6 +499,7 @@ paths mounted via NFS.)
 
 sub sync {
     my $self = shift;
+    $self->pre_sync;
 
     my $command = join " ", $self->{rsync}, $self->{opts};
     $command .= " -v" if $self->{v};
@@ -462,10 +511,12 @@ sub sync {
     if ( system $command ) {
         my $err = $? >> 8;
         require Carp;
-        Carp::carp "Problem during rsync ($err)";
+        Carp::carp( "Problem during rsync ($err)" );
     }
 
-    return $self->check_dot_patch;
+    my $plevel = $self->check_dot_patch;
+    $self->post_sync;
+    return $plevel;
 }
 
 =back
@@ -486,14 +537,15 @@ package Test::Smoke::Syncer::Snapshot;
 
 use Cwd;
 use File::Path;
+use Test::Smoke::Util qw( whereis clean_filename );
 
 =item Test::Smoke::Syncer::Snapshot->new( %args )
 
 This crates the new object. Keys for C<%args>:
 
   * ddir:    destination directory ( ./perl-current )
-  * server:  the server to get the snapshot from ( ftp.funet.fi )
-  * sdir:    server directory ( /pub/languages/perl/snap )
+  * server:  the server to get the snapshot from ( public.activestate.com )
+  * sdir:    server directory ( /pub/apc/perl-current-snap )
   * snapext: the extension used for snapdhots ( tgz )
   * tar:     howto untar ( Archive::Tar or 'gzip -d -c %s | tar x -' )
   * v:       verbose
@@ -519,6 +571,7 @@ and extract the snapshot.
 sub sync {
     my $self = shift;
 
+    $self->pre_sync;
     # we need to have {ddir} before we can save the snapshot
     -d $self->{ddir} or mkpath( $self->{ddir} );
 
@@ -530,7 +583,9 @@ sub sync {
 
     $self->patch_a_snapshot if $self->{patchup};
 
-    return $self->check_dot_patch;
+    my $plevel = $self->check_dot_patch;
+    $self->post_sync;
+    return $plevel;
 }
 
 =item $syncer->_fetch_snapshot( )
@@ -547,46 +602,48 @@ sub _fetch_snapshot {
     return $self->_fetch_snapshot_HTTP if $self->{server} =~ m|^https?://|i;
 
     require Net::FTP;
-    my $ftp = Net::FTP->new( $self->{server}, Debug => 0 ) or do {
+    my $ftp = Net::FTP->new($self->{server}, Debug => 0, Passive => 1) or do {
         require Carp;
-        Carp::carp "[Net::FTP] Can't open $self->{server}: $@";
+        Carp::carp( "[Net::FTP] Can't open $self->{server}: $@" );
         return undef;
     };
 
-    my @login = ( 'anonymous', 'smokers@perl.org' );
+    my @login = ( $self->{ftpusr}, $self->{ftppwd} );
     $ftp->login( @login ) or do {
         require Carp;
-        Carp::carp "[Net:FTP] Can't login( @login )";
+        Carp::carp( "[Net:FTP] Can't login( @login )" );
         return undef;
     };
 
     $self->{v} and print "Connected to $self->{server}\n";
     $ftp->cwd( $self->{sdir} ) or do {
         require Carp;
-        Carp::carp "[Net::FTP] Can't chdir '$self->{sdir}'";
+        Carp::carp( "[Net::FTP] Can't chdir '$self->{sdir}'" );
         return undef;
     };
 
     my $snap_name = $self->{sfile} ||
-                    __find_snap_name( $ftp, $self->{snapext} );
+                    __find_snap_name( $ftp, $self->{snapext}, $self->{v} );
 
     unless ( $snap_name ) {
         require Carp;
-        Carp::carp "Couldn't find a snapshot at $self->{server}$self->{sdir}";
+        Carp::carp("Couldn't find a snapshot at $self->{server}$self->{sdir}");
         return undef;
     }
 
     $ftp->binary(); # before you ask for size!
     my $snap_size = $ftp->size( $snap_name );
-    my $local_snap = File::Spec->catfile( $self->{ddir},
-                                          File::Spec->updir, $snap_name );
+    my $ddir_var = $self->{vms_ddir} ? 'vms_ddir' : 'ddir';
+    my $local_snap = File::Spec->catfile( $self->{ $ddir_var },
+                                          File::Spec->updir,
+                                          clean_filename( $snap_name ) );
     $local_snap = File::Spec->canonpath( $local_snap );
 
     if ( -f $local_snap && $snap_size == -s $local_snap ) {
         $self->{v} and print "Skipping download of '$snap_name'\n";
     } else {
         $self->{v} and print "get ftp://$self->{server}$self->{sdir}/" .
-                             "$snap_name ";
+                             "$snap_name\n as $local_snap ";
         my $l_file = $ftp->get( $snap_name, $local_snap );
         my $ok = $l_file eq $local_snap && $snap_size == -s $local_snap;
         $ok or printf "Error in get(%s) [%d]\n", $l_file || "", 
@@ -612,7 +669,7 @@ sub _fetch_snapshot_HTTP {
 
     unless ( $snap_name ) {
         require Carp;
-        Carp::carp "No snapshot specified for $self->{server}$self->{sdir}";
+        Carp::carp( "No snapshot specified for $self->{server}$self->{sdir}" );
         return undef;
     }
 
@@ -636,7 +693,7 @@ sub _fetch_snapshot_HTTP {
     }
 }
 
-=item __find_snap_name( $ftp, $snapext )
+=item __find_snap_name( $ftp, $snapext[, $verbose] )
 
 [Not a method!]
 
@@ -646,17 +703,21 @@ return the one with the highes number.
 =cut
 
 sub __find_snap_name {
-    my( $ftp, $snapext ) = @_;
+    my( $ftp, $snapext, $verbose ) = @_;
     $snapext ||= 'tgz';
+    $verbose ||= 0;
+    $verbose > 1 and print "Looking for /$snapext\$/\n";
 
     my @list = $ftp->ls();
 
     my $snap_name = ( map $_->[0], sort { $a->[1] <=> $b->[1] } map {
-        [ $_, /^perl\@(\d+)/ ]
+        my( $p_level ) = /^perl[@#_](\d+)/;
+        $verbose > 1 and print "Kept: $_ ($p_level)\n";
+        [ $_, $p_level ]
     } grep {
-    	/^perl\@\d+/ &&
+    	/^perl[@#_]\d+/ &&
     	/$snapext$/ 
-    } @list )[-1];
+    } map { $verbose > 1 and print "Found snapname: $_\n"; $_ } @list )[-1];
 
     return $snap_name;
 }
@@ -674,7 +735,7 @@ sub _extract_snapshot {
 
     unless ( $self->{snapshot} && -f $self->{snapshot} ) {
         require Carp;
-        Carp::carp "No snapshot to be extracted!";
+        Carp::carp( "No snapshot to be extracted!" );
         return undef;
     }
 
@@ -683,11 +744,11 @@ sub _extract_snapshot {
     # Files in the snapshot are relative to the 'perl/' directory,
     # they may need to be moved and that is not easy when you've
     # extracted them in the target directory! so we go updir()
-    my $extract_base = File::Spec->catdir( $self->{ddir},
-                                           File::Spec->updir );
+    my $ddir = $^O eq 'VMS' ? $self->{vms_ddir} : $self->{ddir};
+    my $extract_base = File::Spec->catdir( $ddir, File::Spec->updir );
     chdir $extract_base or do {
         require Carp;
-        Carp::croak "Can't chdir '$extract_base': $!";
+        Carp::croak( "Can't chdir '$extract_base': $!" );
     };
 
     my $snap_base;
@@ -707,7 +768,7 @@ sub _extract_snapshot {
 
     chdir $cwd or do {
         require Carp;
-        Carp::croak "Can't chdir($extract_base) back: $!";
+        Carp::croak( "Can't chdir($extract_base) back: $!" );
     };
 
     if ( $self->{cleanup} & 1 ) {
@@ -730,7 +791,7 @@ sub _extract_with_Archive_Tar {
 
     my $archive = Archive::Tar->new() or do {
         require Carp;
-        Carp::carp "Can't Archive::Tar->new: " . $Archive::Tar::error;
+        Carp::carp( "Can't Archive::Tar->new: " . $Archive::Tar::error );
         return undef;
     };
 
@@ -738,7 +799,7 @@ sub _extract_with_Archive_Tar {
     $archive->read( $self->{snapshot}, 1 );
     $Archive::Tar::error and do {
         require Carp;
-        Carp::carp "Error reading '$self->{snapshot}': ".$Archive::Tar::error;
+        Carp::carp("Error reading '$self->{snapshot}': ".$Archive::Tar::error);
         return undef;
     };
     my @files = $archive->list_files;
@@ -746,7 +807,9 @@ sub _extract_with_Archive_Tar {
     $self->{v} and printf "%d items OK.\n", scalar @files;
 
     ( my $prefix = $files[0] ) =~ s|^([^/]+).+$|$1|;
-    return File::Spec->canonpath( File::Spec->catdir( cwd(), $prefix ) );
+    my $base_dir = File::Spec->canonpath(File::Spec->catdir( cwd(), $prefix ));
+    $self->{v} and print "Snapshot prefix: '$base_dir'\n";
+    return $base_dir;
 }
 
 =item $syncer->_extract_with_external( )
@@ -761,17 +824,21 @@ sub _extract_with_external {
 
     my @dirs_pre = __get_directory_names();
 
-    my $command = sprintf $self->{tar}, $self->{snapshot};
-    $command .= " $self->{snapshot}" if $command eq $self->{tar};
+    if ( $^O ne 'VMS' ) {
+        my $command = sprintf $self->{tar}, $self->{snapshot};
+        $command .= " $self->{snapshot}" if $command eq $self->{tar};
 
-    $self->{v} and print "$command ";
-    if ( system $command ) {
-        my $error = $? >> 8;
-        require Carp;
-        Carp::carp "Error in command: $error";
-        return undef;
-    };
-    $self->{v} and print "OK\n";
+        $self->{v} and print "$command ";
+        if ( system $command ) {
+            my $error = $? >> 8;
+            require Carp;
+            Carp::carp( "Error in command: $error" );
+            return undef;
+        };
+        $self->{v} and print "OK\n";
+    } else {
+        __vms_untargz( $self->{tar}, $self->{snapshot}, $self->{v} );
+    }
 
     # Yes another process can also create directories here!
     # Be careful.
@@ -779,10 +846,45 @@ sub _extract_with_external {
     exists $dirs_post{ $_ } and delete $dirs_post{ $_ }
         foreach @dirs_pre;
     # I'll pick the first one that has 'perl' in it
-    my( $base_dir ) = grep /\bperl/ || /perl\b/ => keys %dirs_post;
-    $base_dir ||= 'perl';
+    my( $prefix ) = grep /\bperl/ || /perl\b/ => keys %dirs_post;
+    my $ddir = $^O eq 'VMS' ? $self->{vms_ddir} : $self->{ddir};
+    $prefix ||= File::Spec->abs2rel( $ddir, cwd() );
 
-    return File::Spec->canonpath( File::Spec->catdir( cwd(), $base_dir ) );
+    my $base_dir = File::Spec->canonpath(File::Spec->catdir( cwd(), $prefix ));
+    $self->{v} and print "Snapshot prefix: '$base_dir'\n";
+    return $base_dir;
+}
+
+=item __vms_untargz( $untargz, $tgzfile, $verbose )
+
+Gunzip and extract the archive in C<$tgzfile> using a small DCL script
+
+=cut
+
+sub __vms_untargz {
+    my( $cmd, $file, $verbose ) = @_;
+    my( $gzip_cmd, $tar_cmd ) = split /\s*\|\s*/, $cmd;
+    my $gzip = $gzip_cmd =~ /^((?:MCR )?\S+)/ ? $1 : 'GZIP';
+    my $tar  = $tar_cmd  =~ /^((?:MCR )?\S+)/
+        ? $1 : (whereis( 'vmstar' ) || whereis( 'tar' ) );
+    my $tar_sw = $verbose ? '-xvf' : '-xf';
+
+    $verbose and print "Writing 'TS-UNTGZ.COM'";
+    local *TMPCOM;
+    open TMPCOM, "> TS-UNTGZ.COM" or return 0;
+    print TMPCOM <<EO_UNTGZ; close TMPCOM or return 0;
+\$! TS-UNTGZ.COM - Generated by Test::Smoke::Syncer
+\$  define/user sys\$output TS-UNTGZ.TAR
+\$  $gzip "-cd" $file
+\$  $tar $tar_sw TS-UNTGZ.TAR
+\$  delete TS-UNTGZ.TAR;*
+EO_UNTGZ
+    $verbose and print " OK\n";
+
+    my $ret = system "\@TS-UNTGZ.COM";
+    1 while unlink "TS-UNTGZ.COM";
+
+    return ! $ret;
 }
 
 =item $syncer->patch_a_snapshot( $patch_number )
@@ -794,12 +896,12 @@ This requires a working B<patch> program.
 You should pass this extra information to
 C<< Test::Smoke::Syncer::Snapshot->new() >>:
 
-  * patchup: should we do this? ( 0 )
-  * pserver: which FTP server? ( ftp2.activestate.com )
-  * pdir:    directory ( /pub/staff/gsar/APC/perl-current-diffs )
-  * unzip:   ( gzip ) [ Compress::Zlib ]
-  * patch:   ( patch )
-  * cleanup: remove patches after applied? ( 1 )
+  * patchup:  should we do this? ( 0 )
+  * pserver:  which FTP server? ( public.activestate.com )
+  * pdir:     directory ( /pub/apc/perl-current-diffs )
+  * unzip:    ( gzip ) [ Compress::Zlib ]
+  * patchbin: ( patch )
+  * cleanup:  remove patches after applied? ( 1 )
 
 =cut
 
@@ -826,22 +928,22 @@ consecutive.
 sub _get_patches {
     my( $self, $patch_number ) = @_;
 
-    my $ftp = Net::FTP->new( $self->{pserver}, Debug => 0 ) or do {
+    my $ftp = Net::FTP->new($self->{pserver}, Debug => 0, Passive => 1) or do {
         require Carp;
-        Carp::carp "[Net::FTP] Can't open '$self->{pserver}': $@";
+        Carp::carp( "[Net::FTP] Can't open '$self->{pserver}': $@" );
         return undef;
     };
 
-    my @user_info = qw( anonymous smokers@perl.org );
+    my @user_info = ( $self->{ftpusr}, $self->{ftppwd} );
     $ftp->login( @user_info ) or do {
         require Carp;
-        Carp::carp "[Net::FTP] Can't login( @user_info )" ;
+        Carp::carp( "[Net::FTP] Can't login( @user_info )" );
         return undef;
     };
 
     $ftp->cwd( $self->{pdir} ) or do {
         require Carp;
-        Carp::carp "[Net::FTP] Can't cd '$self->{pdir}'";
+        Carp::carp( "[Net::FTP] Can't cd '$self->{pdir}'" );
         return undef;
     };
 
@@ -870,6 +972,11 @@ sub _get_patches {
     }
     $ftp->quit;
 
+    @patch_list = map $_->[0] => sort { $a->[1] <=> $b->[1] } map {
+        my( $patch_num ) = /(\d+).gz$/;
+        [ $_, $patch_num ];
+    } @patch_list;
+
     return @patch_list;
 }
 
@@ -891,7 +998,7 @@ sub _apply_patches {
     my $cwd = cwd();
     chdir $self->{ddir} or do {
         require Carp;
-        Carp::croak "Cannot chdir($self->{ddir}): $!";
+        Carp::croak( "Cannot chdir($self->{ddir}): $!" );
     };
 
     require Test::Smoke::Patcher;
@@ -900,15 +1007,15 @@ sub _apply_patches {
         my $patch = $self->_read_patch( $file ) or next;
 
         my $patcher = Test::Smoke::Patcher->new( single => {
-            ddir  => $self->{ddir},
-            patch => $self->{patch},
-            pfile => \$patch,
-            v     => $self->{v},
+            ddir     => $self->{ddir},
+            patchbin => $self->{patchbin},
+            pfile    => \$patch,
+            v        => $self->{v},
         });
         eval { $patcher->patch };
         if ( $@ ) {
              require Carp;
-	     Carp::carp "Error while patching:\n\t$@";
+	     Carp::carp( "Error while patching:\n\t$@" );
              next;
         }
 
@@ -920,7 +1027,7 @@ sub _apply_patches {
     }
     chdir $cwd or do {
         require Carp;
-        Carp::croak "Cannot chdir($cwd) back: $!";
+        Carp::croak( "Cannot chdir($cwd) back: $!" );
     };
 }
 
@@ -940,7 +1047,7 @@ sub _read_patch {
         require Compress::Zlib;
         my $unzip = Compress::Zlib::gzopen( $file, 'rb' ) or do {
             require Carp;
-            Carp::carp "Can't open '$file': $Compress::Zlib::gzerrno";
+            Carp::carp( "Can't open '$file': $Compress::Zlib::gzerrno" );
             return undef;
         };
 
@@ -949,7 +1056,7 @@ sub _read_patch {
  
         unless ( $Compress::Zlib::gzerrno == Compress::Zlib::Z_STREAM_END() ) {
             require Carp;
-            Carp::carp "Error reading '$file': $Compress::Zlib::gzerrno" ;
+            Carp::carp( "Error reading '$file': $Compress::Zlib::gzerrno" );
         }
 
         $unzip->gzclose;
@@ -1002,7 +1109,7 @@ sub __get_directory_names {
 
     local *DIR;
     opendir DIR, $dir or return ();
-    my @dirs = grep -d File::Spec->catdir( $dir, $_ ) => readdir DIR;
+    my @dirs = grep -d File::Spec->catfile( $dir, $_ ) => readdir DIR;
     closedir DIR;
 
     return @dirs;
@@ -1044,19 +1151,27 @@ sub new {
 
 =item $syncer->sync( )
 
-This uses B<Test::Smoke::SourceTree> to do the actual copying.
+This uses B<Test::Smoke::SourceTree> to do the actual copying.  After
+that it will clean up the source-tree (from F<MANIFEST>, but ignoring
+F<MANIFEST.SKIP>!).
 
 =cut
 
 sub sync {
     my $self = shift;
 
+    $self->pre_sync;
     require Test::Smoke::SourceTree;
-    my $tree = Test::Smoke::SourceTree->new( $self->{cdir} );
 
+    my $tree = Test::Smoke::SourceTree->new( $self->{cdir} );
     $tree->copy_from_MANIFEST( $self->{ddir}, $self->{v} );
 
-    return $self->check_dot_patch;
+    $tree = Test::Smoke::SourceTree->new( $self->{ddir} );
+    $tree->clean_from_MANIFEST( 'MANIFEST.SKIP' );
+
+    my $plevel = $self->check_dot_patch;
+    $self->post_sync;
+    return $plevel;
 }
 
 =back
@@ -1096,7 +1211,7 @@ sub new {
     my %args = @_;
     unless ( $args{hdir} ) {
         require Carp;
-        Carp::croak "No source-directory (hdir) specified for " . __PACKAGE__;
+        Carp::croak("No source-directory (hdir) specified for " . __PACKAGE__);
     }
     return bless \%args, $class;
 }
@@ -1110,6 +1225,9 @@ C<sync()> uses the B<File::Find> module to make the hardlink forest in {ddir}.
 sub sync {
     my $self = shift;
 
+    $self->pre_sync;
+    require File::Copy unless $self->{haslink};
+
     -d $self->{ddir} or File::Path::mkpath( $self->{ddir} );
 
     my $source_dir = File::Spec->canonpath( $self->{hdir} );
@@ -1124,7 +1242,9 @@ sub sync {
         } else {
             -e $dest and 1 while unlink $dest;
             $self->{v} > 1 and print "link $_ $dest";
-            my $ok = link $_, $dest;
+            my $ok = $self->{haslink}
+                ? link $_, $dest
+                : File::Copy::copy( $_, $dest );
             if ( $self->{v} > 1 ) {
                 print $ok ? " OK\n" : " $!\n";
             }
@@ -1133,11 +1253,113 @@ sub sync {
 
     $self->clean_from_directory( $source_dir );
 
+    $self->post_sync;
     return $self->check_dot_patch();
 }
 
 =back
 
+=head1 Test::Smoke::Syncer::FTP
+
+This handles syncing by getting the source-tree from ActiveState's APC
+repository. It uses the C<Test::Smoke::FTPClient> that implements a
+mirror function.
+
+=cut
+
+package Test::Smoke::Syncer::FTP;
+
+@Test::Smoke::Syncer::FTP::ISA = qw( Test::Smoke::Syncer );
+
+use File::Spec::Functions;
+
+=head2 Test::Smoke::Syncer::FTP->new( %args )
+
+Known args for this class:
+
+    * ftphost (public.activestate.com)
+    * ftpusr  (anonymous)
+    * ftppwd  (smokers@perl.org)
+    * ftpsdir (/pub/apc/perl-????)
+    * ftpcdir (/pub/apc/perl-????-diffs)
+
+    * ddir
+    * v
+
+=cut
+
+sub new {
+    my $class = shift;
+
+    return bless { @_ }, $class;
+}
+
+=head2 $syncer->sync()
+
+This does the actual syncing:
+
+    * Check {ftpcdir} for the latest changenumber
+    * Mirror 
+
+=cut
+
+sub sync {
+    my $self = shift;
+
+    $self->pre_sync;
+    require Test::Smoke::FTPClient;
+
+    my $fc = Test::Smoke::FTPClient->new( $self->{ftphost}, {
+        v       => $self->{v},
+        passive => $self->{ftppassive},
+        fuser   => $self->{ftpusr},
+        fpwd    => $self->{ftppwd},
+    } );
+
+    $fc->connect;
+
+    $fc->mirror( @{ $self }{qw( ftpsdir ddir )}, 1 ) or return;
+
+    $self->{client} = $fc;
+
+    my $plevel = $self->create_dot_patch;
+    $self->post_sync;
+    return $plevel;
+}
+
+=head2 $syncer->create_dat_patch
+
+This needs to go to the *-diffs directory on APC and find the patch
+whith the highest number, that should be our current patchlevel.
+
+=cut
+
+sub create_dot_patch {
+    my $self = shift;
+    my $ftp = $self->{client}->{client};
+
+    $ftp->cwd( $self->{ftpcdir} );
+    my $plevel = (sort { $b <=> $a } map {
+        s/\.gz$//; $_
+    } grep /\d+\.gz/ => $ftp->ls)[0];
+
+    my $dotpatch = catfile( $self->{ddir}, '.patch' );
+    local *DOTPATH;
+    if ( open DOTPATCH, "> $dotpatch" ) {
+        print DOTPATCH $plevel;
+        close DOTPATCH or do {
+            require Carp;
+            Carp::carp( "Error writing '$dotpatch': $!" );
+        };
+    } else {
+        require Carp;
+        Carp::carp( "Error creating '$dotpatch': $!" );
+    }
+    return $plevel;
+}
+
+1;
+
 =head1 Test::Smoke::Syncer::Forest
 
 This handles syncing by setting up a master directory that is in sync
@@ -1209,15 +1431,15 @@ sub sync {
         my $cwd = Cwd::cwd();
         chdir $self->{fdir} or do {
             require Carp;
-            Carp::croak "Cannot chdir($self->{fdir}) in forest: $!";
+            Carp::croak( "Cannot chdir($self->{fdir}) in forest: $!" );
         };
         system( "$^X regen_headers.pl" ) == 0 or do {
             require Carp;
-            Carp::carp "Error while running 'regen_headers.pl'";
+            Carp::carp( "Error while running 'regen_headers.pl'" );
         };
         chdir $cwd or do {
             require Carp;
-            Carp::croak "Cannot chdir($cwd) back: $!";
+            Carp::croak( "Cannot chdir($cwd) back: $!" );
         };
     }
 
