@@ -1,12 +1,12 @@
 package Test::Smoke::SysInfo;
 use strict;
 
-# $Id: SysInfo.pm 715 2004-07-24 16:10:00Z abeltje $
+# $Id: SysInfo.pm 1026 2007-03-09 14:14:35Z abeltje $
 use vars qw( $VERSION @EXPORT_OK );
-$VERSION = '0.014';
+$VERSION = '0.036';
 
 use base 'Exporter';
-@EXPORT_OK = qw( &sysinfo );
+@EXPORT_OK = qw( &sysinfo &tsuname );
 
 =head1 NAME
 
@@ -18,7 +18,7 @@ Test::Smoke::SysInfo - OO interface to system specific information
 
     my $si = Test::Smoke::SysInfo->new;
 
-    printf "Hostname: %s\n, $si->host;
+    printf "Hostname: %s\n", $si->host;
     printf "Number of CPU's: %s\n", $si->ncpu;
     printf "Processor type: %s\n", $si->cpu_type;   # short
     printf "Processor description: %s\n", $si->cpu; # long
@@ -27,7 +27,11 @@ Test::Smoke::SysInfo - OO interface to system specific information
 or
 
     use Test::Smoke::SysInfo qw( sysinfo );
-    prinft "[%s]\n", sysinfo();
+    printf "[%s]\n", sysinfo();
+
+or
+
+    $ perl -MTest::Smoke::SysInfo=tsuname -le print+tsuname
 
 =head1 DESCRIPTION
 
@@ -36,9 +40,7 @@ smoking.
 
 =head1 METHODS
 
-=over 4
-
-=item Test::Smoke::SysInfo->new( )
+=head2 Test::Smoke::SysInfo->new( )
 
 Dispatch to one of the OS-specific subs.
 
@@ -48,24 +50,28 @@ sub new {
     my $proto = shift;
     my $class = ref $proto ? ref $proto : $proto;
 
-    CASE: {
-        local $_ = $^O;
+    my $chk_os;
+    for $chk_os ( $^O ) {
 
-        /aix/i        && return bless AIX(),     $class;
+        $chk_os =~ /aix/i        && return bless AIX(),     $class;
 
-        /darwin|bsd/i && return bless BSD(),     $class;
+        $chk_os =~ /bsd/i        && return bless BSD(),     $class;
 
-        /hp-?ux/i     && return bless HPUX(),    $class;
+        $chk_os =~ /darwin/i     && return bless Darwin(),  $class;
 
-        /linux/i      && return bless Linux(),   $class;
+        $chk_os =~ /hp-?ux/i     && return bless HPUX(),    $class;
 
-        /irix/i       && return bless IRIX(),    $class;
+        $chk_os =~ /linux/i      && return bless Linux(),   $class;
 
-        /solaris|sunos|osf/i 
-                      && return bless Solaris(), $class;
+        $chk_os =~ /irix/i       && return bless IRIX(),    $class;
 
-        /cygwin|mswin32|windows/i
-                      && return bless Windows(), $class;
+        $chk_os =~ /solaris|sunos|osf/i 
+                                 && return bless Solaris(), $class;
+
+        $chk_os =~ /cygwin|mswin32|windows/i
+                                 && return bless Windows(), $class;
+
+        $chk_os =~ /VMS/         && return bless VMS(),     $class;
     }
     return bless Generic(), $class;
 }
@@ -81,7 +87,7 @@ sub AUTOLOAD {
     return $self->{ "_$method" } if exists $info{ "$method" };
 }
 
-=item __get_os( )
+=head2 __get_os( )
 
 This is the short info string about the Operating System.
 
@@ -91,10 +97,10 @@ sub __get_os {
     require POSIX;
     my $os = join " - ", (POSIX::uname())[0,2];
     $os =~ s/(\S+)/\L$1/;
-    MOREOS: {
-        local $_ = $^O;
+    my $chk_os;
+    for $chk_os ( $^O ) {
 
-        /aix/i             && do {
+        $chk_os =~ /aix/i && do {
             chomp( $os = `oslevel -r` );
             if ( $os =~ m/^(\d+)-(\d+)$/ ) {
                 $os = ( join ".", split //, $1 ) . "/ML$2";
@@ -114,41 +120,49 @@ sub __get_os {
                 $os .= "/$ml";
             }
             $os =~ s/^/AIX - /;
-            last MOREOS;
+            last;
         };
-        /irix/i            && do {
+        $chk_os =~ /irix/i && do {
             chomp( my $osvers = `uname -R` );
             my( $osn, $osv ) = split ' ', $os;
             $osvers =~ s/^$osv\s+(?=$osv)//;
             $os = "$osn - $osvers";
-            last MOREOS;
+            last;
         };
-        /linux/i           && do {
+        $chk_os =~ /linux/i && do {
             my $dist_re = '[-_](?:release|version)\b';
-            my( $distro ) = grep /$dist_re/ => glob( '/etc/*' );
+            my( $distro ) = grep /$dist_re/ && !/\blsb-/ => glob( '/etc/*' );
             last MOREOS unless $distro;
             $distro =~ s|^/etc/||;
             $distro =~ s/$dist_re//i;
             $os .= " [$distro]" if $distro;
-            last MOREOS;
+            last;
         };
-        /solaris|sunos/i   && do {
-            require Config;
-            $os = join " - ", $Config::Config{osname},
-                              $Config::Config{osvers};
+        $chk_os =~ /solaris|sunos|osf/i && do {
+            my( $osn, $osv ) = (POSIX::uname())[0,2];
+            $chk_os =~ /solaris|sunos/i && $osv > 5 and do {
+                $osn = 'Solaris';
+                $osv = '2.' . (split /\./, $osv, 2)[1];
+            };
+            $os = join " - ", $osn, $osv;
+            last;
         };
-        /windows|mswin32/i && do {
+        $chk_os =~ /windows|mswin32/i && do {
             eval { require Win32 };
             $@ and last MOREOS;
             $os = "$^O - " . join " ", Win32::GetOSName();
             $os =~ s/Service\s+Pack\s+/SP/;
-            last MOREOS;
+            last;
+        };
+        $chk_os =~ /vms/i && do {
+            $os = join " - ", (POSIX::uname())[0,3];
+            $os =~ s/(\S+)/\L$1/;
         };
     }
     return $os;
 }
 
-=item __get_cpu_type( )
+=head2 __get_cpu_type( )
 
 This is the short info string about the cpu-type. The L<POSIX> module
 should provide one (portably) with C<POSIX::uname()>.
@@ -160,7 +174,7 @@ sub __get_cpu_type {
     return (POSIX::uname())[4];
 }
 
-=item __get_cpu( )
+=head2 __get_cpu( )
 
 We do not have a portable way to get this information, so assign
 C<_cpu_type> to it.
@@ -169,7 +183,7 @@ C<_cpu_type> to it.
 
 sub __get_cpu { return __get_cpu_type() }
 
-=item __get_hostname( )
+=head2 __get_hostname( )
 
 Get the hostname from C<POSIX::uname()>.
 
@@ -182,7 +196,7 @@ sub __get_hostname {
 
 sub __get_ncpu { return '' }
 
-=item Generic( )
+=head2 Generic( )
 
 Get the information from C<POSIX::uname()>
 
@@ -200,7 +214,7 @@ sub Generic {
 
 }
 
-=item AIX( )
+=head2 AIX( )
 
 Use the L<lsdev> program to find information.
 
@@ -213,8 +227,8 @@ sub AIX {
     my( $info ) = grep /^\S+/ => @lsdev;
     ( $info ) = $info =~ /^(\S+)/;
     $info .= " -a 'state type'";
-    my( $cpu ) = grep /^enable:[^:\s]+/ => `lsattr -E -O -l $info`;
-    ( $cpu ) = $cpu =~ /^enable:([^:\s]+)/;
+    my( $cpu ) = grep /\benable:[^:\s]+/ => `lsattr -E -O -l $info`;
+    ( $cpu ) = $cpu =~ /\benable:([^:\s]+)/;
     $cpu =~ s/\bPowerPC(?=\b|_)/PPC/i;
 
     ( my $cpu_type = $cpu ) =~ s/_.*//;
@@ -236,63 +250,95 @@ sub AIX {
     };
 }
 
-=item HPUX( )
+=head2 HPUX( )
 
-Use the L<ioscan> program to find information.
+Use the L<ioscan>, L<getconf> and L<machinfo> programs to find information.
+
+This routine was contributed by Rich Rauenzahn.
 
 =cut
 
 sub HPUX {
     my $hpux = Generic();
-    local $ENV{PATH} = "/etc:$ENV{PATH}";
-    my $ncpu = grep /^processor/ => `ioscan -fnkC processor`;
-    unless ( $ncpu ) {	# not root?
-        local *SYSLOG;
-        if ( open SYSLOG, "< /var/adm/syslog/syslog.log" ) {
-            while ( <SYSLOG> ) {
-                m/\bprocessor$/ and $ncpu++;
+    my $parisc = 0;
+
+    $hpux->{_os} =~ s/hp-ux/HP-UX/;
+
+    # ioscan is always available
+    $hpux->{_ncpu} = grep /^processor/ => `/usr/sbin/ioscan -knfCprocessor`;
+
+    chomp(my $k64 = `/usr/bin/getconf KERNEL_BITS 2>/dev/null`);
+    $hpux->{_os} .= "/$k64" if(length $k64);
+
+    # For now, unknown cpu_types are set as the Generic
+    chomp(my $cv = `/usr/bin/getconf CPU_VERSION 2>/dev/null`);
+    # see /usr/include/sys/unistd.h for hex values
+    if($cv < 0x20B) {
+#        $hpux->{_cpu_type} = sprintf("Unknown CPU_VERSION 0x%x", $cv);
+    } elsif($cv >= 0x20C && $cv <= 0x20E) {
+        $hpux->{_cpu_type} = "Motorola"; # You have an antique
+    } elsif($cv <= 0x2FF) {
+        $hpux->{_cpu_type} = "PA-RISC";
+        $hpux->{_cpu_type} = "PA-RISC1.0" if $cv == 0x20B;
+        $hpux->{_cpu_type} = "PA-RISC1.1" if $cv == 0x210;
+        $hpux->{_cpu_type} = "PA-RISC1.2" if $cv == 0x211;
+        $hpux->{_cpu_type} = "PA-RISC2.0" if $cv == 0x214;
+        $parisc++;
+    } elsif($cv == 0x300) {
+#        $hpux->{_cpu_type} = "Itanium,archrev0";
+        $hpux->{_cpu_type} = "ia64";
+    } else {
+#        $hpux->{_cpu_type} = sprintf("Unknown CPU_VERSION 0x%x", $cv);
+    }
+
+    if ( $parisc ) {
+        my( @cpu, $lst );
+        chomp( my $model = `model` );
+        ( my $m = $model ) =~ s:.*/::;
+        local *LST; my $f;
+        foreach $f (qw( /usr/sam/lib/mo/sched.models
+                        /opt/langtools/lib/sched.models )) {
+            if ( open LST, "< $f" ) {
+                @cpu = grep m/$m/i => <LST>;
+                close LST;
+                @cpu and last;
             }
         }
-    }
-    $hpux->{_ncpu} = $ncpu;
-    # http://wtec.cup.hp.com/~cpuhw/hppa/hversions.html
-    my( @cpu, $lst );
-    chomp( my $model = `model` );
-    ( my $m = $model ) =~ s:.*/::;
-    local *LST;
-    open LST, "< /usr/sam/lib/mo/sched.models" and
-	@cpu = grep m/$m/i, <LST>;
-    close LST;
-
-    @cpu == 0 && open LST, "< /opt/langtools/lib/sched.models" and
-	@cpu = grep m/$m/i, <LST>;
-    close LST;
-
-    if (@cpu == 0 && open LST, "echo 'sc product cpu;il' | /usr/sbin/cstm |") {
-        while (<$lst>) {
-            s/^\s*(PA)\s*(\d+)\s+CPU Module.*/$m 1.1 $1$2/ or next;
-            $2 =~ m/^8/ and s/ 1.1 / 2.0 /;
-            push @cpu, $_;
+        if (@cpu == 0 && open my $lst,
+                              "echo 'sc product cpu;il' | /usr/sbin/cstm |") {
+            while (<$lst>) {
+                s/^\s*(PA)\s*(\d+)\s+CPU Module.*/$m 1.1 $1$2/ or next;
+                $2 =~ m/^8/ and s/ 1.1 / 2.0 /;
+                push @cpu, $_;
+            }
+        }
+        if (@cpu and $cpu[0] =~ m/^\S+\s+(\d+\.\d+[a-z]?)\s+(\S+)/) {
+             my( $arch, $cpu ) = ( "PA-RISC$1", $2 );
+             $hpux->{_cpu} = $cpu;
+             chomp( my $hw3264 = 
+                    `/usr/bin/getconf HW_32_64_CAPABLE 2>/dev/null` );
+            (my $osvers = $hpux->{_os}) =~ s/.*[AB]\.//;
+            $osvers =~ s{/.*}{};
+            $osvers <= 10.20 and $hw3264 = 0;
+            if ( $hw3264 == 1 ) {
+                $hpux->{_cpu_type} = $arch . "/64";
+            } elsif ( $hw3264 == 0 ) {
+                $hpux->{_cpu_type} = $arch . "/32";
+            }
+        }
+    } else {
+        my $machinfo = `/usr/contrib/bin/machinfo`;
+        if( $machinfo =~ m/processor model:\s+(\d+)\s+(.*)/ ) {
+            $hpux->{_cpu} = $2;
+        }
+        if( $machinfo =~ m/Clock\s+speed\s+=\s+(.*)/ ) {
+            $hpux->{_cpu} .= "/$1";
         }
-    }
-    $hpux->{_os} =~ s/ B\.(\d+)/ $1/;
-    my $os_r = $1;
-
-    if ($os_r >= 11) {
-       chomp( my $k64 = `getconf KERNEL_BITS` );
-       $k64 and $hpux->{_os} .= "/$k64";
-    }
-
-    if ($cpu[0] =~ m/^\S+\s+(\d+\.\d+)\s+(\S+)/) {
-        my( $arch, $cpu ) = ("PA-$1", $2);
-        $hpux->{_cpu} = $cpu;
-        $hpux->{_cpu_type} = $os_r >= 11 && `getconf HW_32_64_CAPABLE` =~ m/^1/
-            ? "$arch/64" : "$arch/32";
     }
     return $hpux;
 }
 
-=item BSD( )
+=head2 BSD( )
 
 Use the L<sysctl> program to find information.
 
@@ -303,7 +349,7 @@ sub BSD {
 
     my $sysctl_cmd = -x '/sbin/sysctl' ? '/sbin/sysctl' : 'sysctl';
 
-    my %extra = ( cpufrequency => undef );
+    my %extra = ( cpufrequency => undef, cpuspeed => undef );
     my @e_args = map {
         /^hw\.(\w+)\s*[:=]/; $1
     } grep /^hw\.(\w+)/ && exists $extra{ $1 } => `$sysctl_cmd -a hw`; 
@@ -312,13 +358,18 @@ sub BSD {
         chomp( $sysctl{ $name } = `$sysctl_cmd hw.$name` );
         $sysctl{ $name } =~ s/^hw\.$name\s*[:=]\s*//;
     }
+    $sysctl{machine} and $sysctl{machine} =~ s/Power Macintosh/macppc/;
 
     my $cpu = $sysctl{model};
-    exists $sysctl{cpufrequency}
-        and $cpu .= sprintf " (%.0f MHz)", $sysctl{cpufrequency}/1_000_000;
+
+    if ( exists $sysctl{cpuspeed} ) {
+        $cpu .= sprintf " (%.0f MHz)", $sysctl{cpuspeed};
+    } elsif ( exists $sysctl{cpufrequency} ) {
+        $cpu .= sprintf " (%.0f MHz)", $sysctl{cpufrequency}/1_000_000;
+    }
 
     return {
-        _cpu_type => $sysctl{machine} || __get_cpu_type(),
+        _cpu_type => ($sysctl{machine} || __get_cpu_type()),
         _cpu      => $cpu || __get_cpu,
         _ncpu     => $sysctl{ncpu},
         _host     => __get_hostname(),
@@ -326,7 +377,44 @@ sub BSD {
     };
 }
 
-=item IRIX( )
+=head2 Darwin( )
+
+If the L<system_profiler> program is accessible (meaning that this is
+Mac OS X), use it to find information; otherwise treat as L</BSD>.
+
+This sub was donated by Dominic Dunlup.
+
+=cut
+
+sub Darwin {
+    my $system_profiler_output;
+    {
+        local $^W = 0;
+	$system_profiler_output =
+	    `/usr/sbin/system_profiler -detailLevel mini SPHardwareDataType`;
+    }
+    return BSD() unless $system_profiler_output;
+
+    my %system_profiler;
+    $system_profiler{$1} = $2
+	while $system_profiler_output =~ m/^\s*([\w ]+):\s+(.+)$/gm;
+
+    $system_profiler{'CPU Type'} =~ s/PowerPC\s*(\w+).*/macppc$1/;
+    $system_profiler{'CPU Speed'} =~ 
+	s/(\d+(?:\.\d+)?)\s*GHz/sprintf("%d MHz", $1 * 1000)/e;
+
+    my $model = $system_profiler{'Machine Name'} ||
+                $system_profiler{'Machine Model'};
+    return {
+        _cpu_type => ($system_profiler{'CPU Type'} || __get_cpu_type()),
+        _cpu      => ("$model ($system_profiler{'CPU Speed'})" || __get_cpu),
+        _ncpu     => $system_profiler{'Number Of CPUs'},
+        _host     => __get_hostname(),
+        _os       => __get_os() . " (Mac OS X)",
+    };
+}
+
+=head2 IRIX( )
 
 Use the L<hinv> program to get the system information.
 
@@ -351,7 +439,7 @@ sub IRIX {
 
 }
 
-=item __from_proc_cpuinfo( $key, $lines )
+=head2 __from_proc_cpuinfo( $key, $lines )
 
 Helper function to get information from F</proc/cpuinfo>
 
@@ -365,7 +453,7 @@ sub __from_proc_cpuinfo {
     return $value;
 }
 
-=item Linux( )
+=head2 Linux( )
 
 Use the C</proc/cpuinfo> preudofile to get the system information.
 
@@ -405,7 +493,7 @@ sub Linux {
     };
 }
 
-=item Linux_sparc( )
+=head2 Linux_sparc( )
 
 Linux on sparc architecture seems too different from intel
 
@@ -440,7 +528,7 @@ sub Linux_sparc {
     };
 }
 
-=item Linux_ppc( )
+=head2 Linux_ppc( )
 
 Linux on ppc architecture seems too different from intel
 
@@ -459,6 +547,11 @@ sub Linux_ppc {
         my %info = map {
             ( $_ => __from_proc_cpuinfo( $_, \@cpu_info ) );
         } @parts;
+        if ($info{detected} = __from_proc_cpuinfo( 'detected as', \@cpu_info )){
+            $info{detected} =~ s/.*(\b.+Mac G\d).*/$1/;
+            $info{machine} = $info{detected};
+        }
+        
         $cpu = sprintf "%s %s (%s)", map $info{ $_ } => @parts;
     } else {
         $cpu = __get_cpu();
@@ -473,21 +566,45 @@ sub Linux_ppc {
     };
 }
 
-=item Solaris( )
+=head2 Solaris( )
 
 Use the L<psrinfo> program to get the system information.
+Used also in Tru64 (osf).
 
 =cut
 
 sub Solaris {
 
-    my( $psrinfo ) = grep /the .* operates .* mhz/ix => `psrinfo -v`;
-    my( $type, $speed ) = $psrinfo =~ /the (\w+) processor.*at (\d+) mhz/i;
-    $type =~ s/(v9)$/ $1 ? "-LP64" : "-LP32"/e;
-    my( $cpu_line ) = grep /\s+$speed\s+MHz\s+/i => `prtdiag`;
-    ( my $cpu = ( split " ", $cpu_line )[4] ) =~ s/.*,//;
-    $cpu .= " (${speed}MHz)";
-    my $ncpu = grep /on-line/ => `psrinfo`;
+    local $ENV{PATH} = "/usr/sbin:$ENV{PATH}";
+
+    my @psrinfo = `psrinfo -v`;
+    my( $psrinfo ) = grep /the .* operates .* [gm]hz/ix => @psrinfo;
+    my( $type, $speed, $magnitude ) =
+        $psrinfo =~ /the (.+) processor.*at (.+?)\s*([GM]hz)/i;
+
+    $type =~ s/(v9)$/ $1 ? "64" : ""/e;
+
+    my $cpu = __get_cpu();
+    if ( -d "/usr/platform" ) { # Solaris but not OSF/1.
+        chomp( my $platform = `uname -i` );
+        my $pfpath = "/usr/platform/$platform/sbin/prtdiag";
+        if ( -x "$pfpath" ) { # Not on Solaris-x86
+            my $prtdiag = `$pfpath`;
+            ( $cpu ) = $prtdiag =~ /^System .+\(([^\s\)]+)/;
+            unless ( $cpu ) {
+                my($cpu_line) = grep /\s+on-?line\s+/i => split /\n/, $prtdiag;
+                ( $cpu = ( split " ", $cpu_line )[4] ) =~ s/.*,//;
+            }
+            $cpu .= " ($speed$magnitude)";
+        } else {
+            $cpu .= " ($speed$magnitude)";
+        }
+    } elsif (-x "/usr/sbin/sizer") { # OSF/1.
+        $cpu = $type;
+        chomp( $type = `sizer -implver` );
+    }
+
+    my $ncpu = grep /on-?line/ => `psrinfo`;
 
     return {
         _cpu_type => $type,
@@ -498,7 +615,7 @@ sub Solaris {
     };
 }
 
-=item Windows( )
+=head2 Windows( )
 
 Use the C<%ENV> hash to find information. Fall back on the *::Generic
 values if these values have been unset or are unavailable (sorry I do
@@ -509,10 +626,8 @@ Use L<Win32::TieRegistry> if available to get better information.
 =cut
 
 sub Windows {
+    my( $cpu_type, $cpu, $ncpu );
 
-    my $cpu_type = $ENV{PROCESSOR_ARCHITECTURE};
-    my $cpu      = $ENV{PROCESSOR_IDENTIFIER};
-    my $ncpu     = $ENV{NUMBER_OF_PROCESSORS};
     eval { require Win32::TieRegistry };
     unless ( $@ ) {
         Win32::TieRegistry->import();
@@ -532,15 +647,44 @@ sub Windows {
     }
 
     return {
-        _cpu_type => $cpu_type,
-        _cpu      => $cpu,
-        _ncpu     => $ncpu,
+        _cpu_type => ( $cpu_type || $ENV{PROCESSOR_ARCHITECTURE} ),
+        _cpu      => ( $cpu || $ENV{PROCESSOR_IDENTIFIER} ),
+        _ncpu     => ( $ncpu || $ENV{NUMBER_OF_PROCESSORS} ),
         _host     => __get_hostname(),
         _os       => __get_os(),
     };
 }
 
-=item sysinfo( )
+=head2 VMS()
+
+Use some VMS specific stuff to get system information. These were
+suggested by Craig Berry.
+
+=cut
+
+sub VMS {
+    my $vms = Generic();
+
+#    my $myname = $vms->{_host};
+#    my @cpu_brief = `SHOW CPU/BRIEF`;
+#    my( $sysline ) = grep /$myname,(?:\s+a)?\s+/i => @cpu_brief;
+#    my( $cpu ) = $sysline =~ /$myname,(?:\s+a)?\s+(.+)/i;
+#    my $ncpu = grep /^CPU \d+/ && /\bstate\b/i && /\bRUN\b/i => @cpu_brief;
+
+    my %map = ( 
+        cpu      => 'HW_NAME',
+        cpu_type => 'ARCH_NAME',
+        ncpu     => 'ACTIVECPU_CNT'
+    );
+    for my $key ( keys %map ) {
+        my $cmd_out = `write sys\$output f\$getsyi("$map{$key}")`;
+        chomp( $vms->{ "_$key" } = $cmd_out );
+    }
+
+    return $vms;
+}
+
+=head2 sysinfo( )
 
 C<sysinfo()> returns a string with C<host>, C<os> and C<cpu_type>.
 
@@ -554,9 +698,38 @@ sub sysinfo {
     return join " ", @{ $si }{ map "_$_" => @fields };
 }
 
-1;
+=head2 tsuname( @args )
 
-=back
+This class gathers most of the C<uname(1)> info, make a comparable
+version. Takes almost the same arguments:
+
+    a for all (can be omitted)
+    n for nodename
+    s for os name and version
+    m for cpu name
+    c for cpu count
+    p for cpu_type
+
+=cut
+
+sub tsuname {
+    my @args = map split() => @_;
+
+    my @sw = qw( n s m c p );
+    my %sw = ( n => '_host', s => '_os',
+               m => '_cpu', c => '_ncpu', p => '_cpu_type' );
+
+    @args = grep exists $sw{ $_ } => @args;
+    @args or @args = ( 'a' );
+    grep( /a/ => @args ) and @args = @sw;
+    my %show = map +( $_ => undef ) => grep exists $sw{ $_ } => @args;
+    @args = grep exists $show{ $_ } => @sw;
+
+    my $si = Test::Smoke::SysInfo->new;
+    return join " ", @{ $si }{ @sw{ @args } };
+}
+
+1;
 
 =head1 SEE ALSO
 
@@ -564,10 +737,11 @@ L<Test::Smoke::Smoker>, L<Test::Smoke::Reporter>
 
 =head1 COPYRIGHT
 
-(c) 2002-2003, Abe Timmerman <abeltje@cpan.org> All rights reserved.
+(c) 2002-2006, Abe Timmerman <abeltje@cpan.org> All rights reserved.
 
 With contributions from Jarkko Hietaniemi, Merijn Brand, Campo
-Weijerman, Alan Burlison, Allen Smith, Alain Barbet.
+Weijerman, Alan Burlison, Allen Smith, Alain Barbet, Dominic Dunlop,
+Rich Rauenzahn.
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
